﻿;==================================================================
; Rosetta Task: AKS test for primes
; 
; The AKS algorithm for testing whether a number is prime is a polynomial-time algorithm based on an 
; elementary theorem about Pascal triangles.
; 
; The theorem on which the test is based can be stated as follows:
; 
; a number p is prime if and only if all the coefficients of the polynomial expansion of
; (x-1)^p - (x^p - 1)
; are divisible by p.
; 
; For example, trying p=3:
; 
; (x-1)^3 - (x^3 - 1) 
;    = (x^3 -3x^2 +3x -1) - (x^3 - 1)
;    = -3x^2 +3x
;
; And all the coefficients are divisible by 3 so 3 is prime.
; Note:
; This task is not the AKS primality test. It is an inefficient exponential time algorithm discovered in the late 1600s  
; And used as an introductory lemma in the AKS derivation.
; The task
; Create a function/subroutine/method that given p generates the coefficients of the expanded polynomial representation of (x-1)^p.
; Use the function to show here the polynomial expansions of (x-1)^p for p in the range 0 to at least 7, inclusive.
; Use the previous function in creating another function that when given p returns whether p is prime using the theorem.
; Use your test to generate a List of all primes under 35.
; As a stretch goal, generate all primes under 50 (Needs greater than 31 bit integers).
;==================================================================

 
EnableExplicit
Define vzr.b = -1, vzc.b = ~vzr, nMAX.i = 10, n , k
 
Procedure coeff(nRow.i, Array pd.i(2))
  Define.i n, k
  For n=1 To nRow
    For k=0 To n
      If k=0 Or k=n : pd(n,k)=1 : Continue : EndIf
      pd(n,k)=pd(n-1,k-1)+pd(n-1,k)
    Next
  Next
EndProcedure
 
Procedure.b isPrime(n.i, Array pd.i(2))
  Define.i m
  For m=1 To n-1
    If Not pd(n,m) % n = 0 : ProcedureReturn #False : EndIf
  Next
  ProcedureReturn #True
EndProcedure
 
Dim pd.i(nMAX,nMAX)
pd(0,0)=1 : coeff(nMAX, pd())
OpenConsole()
 
For n=0 To nMAX
  Print(RSet(Str(n),3,Chr(32))+": ")
  If vzr : Print("+") : Else : Print("-") : EndIf
  For k=0 To n    
    If k>0 : If vzc : Print("+") : Else : Print("-") : EndIf : vzc = ~vzc : EndIf
    Print(RSet(Str(pd(n,k)),3,Chr(32))+Space(3))    
  Next
  PrintN("")
  vzr = ~vzr : vzc = ~vzr  
Next
PrintN("")
 
nMAX=50 : Dim pd.i(nMAX,nMAX)
Print("Primes n<=50 : ") : coeff(nMAX, pd())
For n=2 To 50
  If isPrime(n,pd()) : Print(Str(n)+Space(2)) : EndIf
Next
Input()

;==================================================================
 
; Output:
;   0: +  1
;   1: -  1   +  1
;   2: +  1   -  2   +  1
;   3: -  1   +  3   -  3   +  1
;   4: +  1   -  4   +  6   -  4   +  1
;   5: -  1   +  5   - 10   + 10   -  5   +  1
;   6: +  1   -  6   + 15   - 20   + 15   -  6   +  1
;   7: -  1   +  7   - 21   + 35   - 35   + 21   -  7   +  1
;   8: +  1   -  8   + 28   - 56   + 70   - 56   + 28   -  8   +  1
;   9: -  1   +  9   - 36   + 84   -126   +126   - 84   + 36   -  9   +  1
;  10: +  1   - 10   + 45   -120   +210   -252   +210   -120   + 45   - 10   +  1
; 
; 
; Primes n<=50 : 2  3  5  7  11  13  17  19  23  29  31  37  41  43  47
;==================================================================


; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 26
; Folding = -
; EnableUnicode
; EnableXP