﻿;==================================================================
; Rosetta Task: Amb

; Define and give an example of the Amb operator.
; 
; The Amb operator expresses nondeterminism. This doesn't refer to randomness (as in "nondeterministic universe") 
; but is closely related to the term as it is used in automata theory ("non-deterministic finite automaton").
; 
; The Amb operator takes a variable number of expressions (or values if that's simpler in the language) and yields a 
; correct one which will satisfy a constraint in some future computation, thereby avoiding failure.
; 
; Problems whose solution the Amb operator naturally expresses can be approached With other tools, such as explicit 
; nested iterations over data sets, or with pattern matching. By contrast, the Amb operator appears integrated into 
; the language. Invocations of Amb are not wrapped in any visible loops or other search patterns; they appear to be independent.
; 
; Essentially Amb(x, y, z) splits the computation into three possible futures: a future in which the value x is yielded, a 
; future in which the value y is yielded and a future in which the value z is yielded. The future which leads to a successful 
; subsequent computation is chosen. The other "parallel universes" somehow go away. Amb called with no arguments fails.
; 
; For simplicity, one of the domain values usable with Amb may denote failure, If that is convenient.
; For instance, it is convenient if a Boolean false denotes failure, so that Amb(false) fails, and thus constraints can 
; be expressed using Boolean expressions like Amb(x * y == 8) which unless x And y add to four.
; 
; A pseudo-code program which satisfies this constraint might look like:
; 
; let x = Amb(1, 2, 3)
; let y = Amb(7, 6, 4, 5)
; Amb(x * y = 8)
; print x, y
; The output is 2 4 because Amb(1, 2, 3) correctly chooses the future in which x has value 2, Amb(7, 6, 4, 5) chooses 4 and 
; consequently Amb(x * y = 8) produces a success.
; 
; Alternatively, failure could be represented using strictly Amb():
; 
; unless x * y = 8 do Amb()
; or else Amb could take the form of two operators or functions: one for producing values and one for enforcing constraints:
; 
; let x = Ambsel(1, 2, 3)
; let y = Ambsel(4, 5, 6)
; Ambassert(x * y = 8)
; print x, y
; where Ambassert behaves like Amb() if the boolean expression is false, otherwise it allows the future computation to take place,
; without yielding any value.
; 
; The task is to somehow implement Amb, and demonstrate it with a program which chooses one word from each of the following 
; four sets of character strings to generate a four-word sentence:
; 
; "the" "that" "a"
; "frog" "elephant" "thing"
; "walked" "treaded" "grows"
; "slowly" "quickly"
; The constraint to be satisfied is that the last character of each word (other than the last) is the same as the first character of its successor.
; 
; The only successful sentence is "that thing grows slowly"; other combinations do not satisfy the constraint and thus fail.
; 
; The goal of this task isn't to simply process the four lists of words with explicit, deterministic program flow such as nested 
; iteration, to trivially demonstrate the correct output. The goal is to implement the Amb operator, or a facsimile thereof that is 
; possible within the language limitations.
;==================================================================

Procedure Words_Ok(String1.s, String2.s)
  If Mid(String1,Len(String1),1)=Mid(String2,1,1)
    ProcedureReturn #True
  EndIf
  ProcedureReturn #False
EndProcedure
 
Procedure.s Amb(Array A.s(1), Array B.s(1), Array C.s(1), Array D.s(1))
  Protected a, b, c, d
  For a=0 To ArraySize(A())
    For b=0 To ArraySize(B())
      For c=0 To ArraySize(C())
        For d=0 To ArraySize(D())
          If Words_Ok(A(a),B(b)) And Words_Ok(B(b),C(c)) And Words_Ok(C(c),D(d))
            ProcedureReturn A(a)+" "+B(b)+" "+C(c)+" "+D(d)
          EndIf
        Next
      Next
    Next
  Next
  ProcedureReturn ""   ; Empty string, e.g. fail
EndProcedure
 
If OpenConsole()
  Define Text.s
  Dim Set1.s(2)
  Dim Set2.s(2)
  Dim Set3.s(2)
  Dim Set4.s(1)
 
  Set1(0)="the":    set1(1)="that":     set1(2)="a"
  Set2(0)="frog":   set2(1)="elephant": set2(2)="thing" 
  Set3(0)="walked": set3(1)="treaded":  set3(2)="grows" 
  Set4(0)="slowly": set4(1)="quickly"
 
  text=Amb(set1(),set2(),Set3(),set4())
  If Text<>""
    PrintN("Correct sentence would be,"+#CRLF$+Text)
  Else
    PrintN("Failed to fine a correct sentence.")
  EndIf
  PrintN(#CRLF$+#CRLF$+"Press ENTER to exit."): Input()
  CloseConsole()
EndIf
;==================================================================

; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 51
; FirstLine = 24
; Folding = -
; EnableUnicode
; EnableXP