﻿;==================================================================
; Rosetta Task: Averages - Simple moving average

; Computing the simple moving average of a series of numbers.
; 
; The task is to: 
; Create a stateful function/class/instance that takes a period and returns a routine that takes a number 
; as argument and returns a simple moving average of its arguments so far.
;
; Description: 
; A simple moving average is a method for computing an average of a stream of numbers by 
; only averaging the last P numbers from the stream, where P is known as the period. It can be implemented 
; by calling an initialing routine with P As its argument, I(P), which should then return a routine that when 
; called with individual, successive members of a stream of numbers, computes the mean of (up to), the last P of them, lets call this SMA().
; 
; The word stateful in the task description refers to the need For SMA() to remember certain information 
;  between calls to it:
;     - The period, P
;     - An ordered container of at least the last P numbers from each of its individual calls.
;
; Stateful also means that successive calls to I(), the initializer, should return separate routines that do not 
; share saved state so they could be used on two independent streams of Data.
;==================================================================

Procedure.d SMA(Number, Period=0)
  Static P
  Static NewList L()
  Protected Sum=0
  If Period<>0
    P=Period
  EndIf
  LastElement(L())
  AddElement(L())
  L()=Number
  While ListSize(L())>P
    FirstElement(L())
    DeleteElement(L(),1)
  Wend
  ForEach L()
    sum+L()
  Next 
  ProcedureReturn sum/ListSize(L())
EndProcedure
; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 20
; Folding = -
; EnableUnicode
; EnableXP