﻿;==================================================================
; Rosetta Task: Balanced brackets
; 
; Generate a string With {N} opening brackets (“[”) And {N} closing brackets (“]”), in some arbitrary order.
;
; Determine whether the generated string is balanced; that is, whether it consists entirely of pairs of 
; opening/closing brackets (in that order), none of which mis-nest.
;
; Examples:
;    (empty)   OK
;    []        OK   ][        Not OK
;    [][]      OK   ][][      Not OK
;    [[][]]    OK   []][[]    Not OK
;
;==================================================================

Procedure.s Generate(N)
  For i=1 To N
    sample$+"[]"
  Next
  For i=Len(sample$)-1 To 2 Step -1
    r=Random(i-1)+1
    If r<>i
      a.c=PeekC(@sample$+r*SizeOf(Character))
      b.c=PeekC(@sample$+i*SizeOf(Character))
      PokeC(@sample$+r*SizeOf(Character), b)
      PokeC(@sample$+i*SizeOf(Character), a) 
    EndIf
  Next
  ProcedureReturn sample$
EndProcedure
 
Procedure Balanced(String$)
  Protected *p.Character, cnt
  *p=@String$
  While *p\c
    If *p\c='['
      cnt+1
    ElseIf *p\c=']'
      cnt-1
      If cnt<0: Break: EndIf
    EndIf
    *p+SizeOf(Character)
  Wend
  If cnt=0
    ProcedureReturn #True
  EndIf
EndProcedure
 
;- Test code
OpenConsole()
For i=1 To 5
  TestString$ = Generate(i)
  Print(TestString$)
  If Balanced(TestString$)
    PrintN(" is balanced.")
  Else
    PrintN(" is not balanced")
  EndIf
Next
Input()

; Output sample
; 
;  [] is balanced.
;  [[]] is balanced.
;  [[[]]] is balanced.
;  [][]]][[ is Not balanced
;  [][][][]][ is Not balanced
; IDE Options = PureBasic 5.42 Beta 1 LTS (Windows - x86)
; CursorPosition = 13
; Folding = -
; EnableUnicode
; EnableXP