﻿;==================================================================
; Rosetta Task: Binary search
;
; A binary search divides a range of values into halves, and continues to narrow down the field of search 
; until the unknown value is found. It is the classic example of a "divide and conquer" algorithm.
; 
; As an analogy, consider the children's game "guess a number." The scorer has a secret number, and 
; will only tell the player if their guessed number is higher than, lower than, or equal to the secret number. 
; The player then uses this information to guess a new number.
; 
; As the player, an optimal strategy For the general case is to start by choosing the range's midpoint 
; as the guess, and then asking whether the guess was higher, lower, or equal to the secret number. 
; If the guess was too high, one would select the point exactly between the range midpoint and the 
; beginning of the range. If the original guess was too low, one would ask about the point exactly 
; between the range midpoint and the end of the range. This process repeats until one has reached the secret number.
; 
; The Task
; 
; Given the starting point of a range, the ending point of a range, and the "secret value", implement a binary 
; search through a sorted integer array for a certain number. Implementations can be recursive or iterative 
; (both If you can). Print out whether or not the number was in the array afterwards. If it was, print the index also.
; 
; There are several binary search algorithms commonly seen. They differ by how they treat multiple values 
; equal to the given value, and whether they indicate whether the element was found or not. For 
; completeness we will present pseudocode for all of them.
; 
; All of the following code examples use an "inclusive" upper bound (i.e. high = N-1 initially). 
; Any of the examples can be converted into an equivalent example using "exclusive" upper bound 
; (i.e. high = N initially) by making the following simple changes (which simply increase high by 1):
; 
; change high = N-1 To high = N
; change high = mid-1 To high = mid
; (For recursive algorithm) change If (high < low) To If (high <= low)
; (For iterative algorithm) change While (low <= high) To While (low < high)
; Traditional algorithm
; The algorithms are as follows (from Wikipedia). The algorithms return the index of some element that 
; equals the given value (If there are multiple such elements, it returns some arbitrary one). It is also possible, 
; when the element is not found, to return the "insertion point" for it (the index that the value would have 
; if it were inserted into the array).

;==================================================================

; Note: Both recursive and iterative procedures are included and called in the code below.

#Recursive = 0 ;recursive binary search method
#Iterative = 1 ;iterative binary search method
#NotFound = -1 ;search result if item not found
 
;Recursive
Procedure  R_BinarySearch(Array a(1), value, low, high)
  Protected mid
  If high < low
    ProcedureReturn #NotFound
  EndIf 
 
  mid = (low + high) / 2
  If a(mid) > value
    ProcedureReturn R_BinarySearch(a(), value, low, mid - 1)
  ElseIf a(mid) < value
    ProcedureReturn R_BinarySearch(a(), value, mid + 1, high)
  Else
    ProcedureReturn mid
  EndIf 
EndProcedure
 
;Iterative
Procedure I_BinarySearch(Array a(1), value, low, high)
  Protected mid
  While low <= high
    mid = (low + high) / 2
    If a(mid) > value            
      high = mid - 1
    ElseIf a(mid) < value
      low = mid + 1
    Else
      ProcedureReturn mid
    EndIf
  Wend
 
  ProcedureReturn #NotFound
EndProcedure
 
Procedure search (Array a(1), value, method)
  Protected idx
 
  Select method
    Case #Iterative
      idx = I_BinarySearch(a(), value, 0, ArraySize(a()))
    Default
      idx = R_BinarySearch(a(), value, 0, ArraySize(a()))
  EndSelect
 
  Print("  Value " + Str(Value))
  If idx < 0
    PrintN(" not found")
  Else
    PrintN(" found at index " + Str(idx))
  EndIf
EndProcedure
 
 
#NumElements = 9 ;zero based count
Dim test(#NumElements)
 
DataSection
  Data.i 2, 3, 5, 6, 8, 10, 11, 15, 19, 20
EndDataSection
 
;fill the test array
For i = 0 To #NumElements		
  Read test(i)
Next
 
 
If OpenConsole()
 
  PrintN("Recursive search:")
  search(test(), 4, #Recursive)
  search(test(), 8, #Recursive)
  search(test(), 20, #Recursive)
 
  PrintN("")
  PrintN("Iterative search:")
  search(test(), 4, #Iterative)
  search(test(), 8, #Iterative)
  search(test(), 20, #Iterative)
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit")
  Input()
  CloseConsole()
EndIf
; Sample output:
; 
; Recursive search:
;   Value 4 Not found
;   Value 8 found at index 4
;   Value 20 found at index 9
; 
; Iterative search:
;   Value 4 Not found
;   Value 8 found at index 4
;   Value 20 found at index 9

; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 42
; Folding = -
; EnableUnicode
; EnableXP