﻿;==================================================================
; Rosetta Task: Brownian tree

; Generate and draw a Brownian Tree.
; 
; A Brownian Tree is generated as a result of an initial seed, followed by the interaction of two processes.
; 
; The initial "seed" is placed somewhere within the field. Where is not particularly important; it could be 
; randomized, or it could be a fixed point.
; Particles are injected into the field, and are individually given a (typically random) motion pattern.
; When a particle collides with the seed or tree, its position is fixed, and it's considered to be part of the tree.
; Because of the lax rules governing the random nature of the particle's placement and motion, no two 
; resulting trees are really expected to be the same, or even necessarily have the same general shape.

;==================================================================

#Window1   = 0
#Image1    = 0
#ImgGadget = 0
 
#NUM_PARTICLES = 3000
#width         = 200
#height        = 200
#xmax          = #width  -3
#ymax          = #height -3
Define.i Event ,i ,x,y
 
If OpenWindow(#Window1, 0, 0, #width, #height, "Brownian Tree PureBasic Example", #PB_Window_SystemMenu )
   If CreateImage(#Image1, #width, #height)
      ImageGadget(#ImgGadget, 0, 0, #width, #height, ImageID(#Image1))
      StartDrawing(ImageOutput(#Image1))
      FrontColor($FFFFFF)
      Plot( Random(#xmax) , Random(#ymax ))
      StopDrawing()
      SetGadgetState(#ImgGadget, ImageID(#Image1))
      For i = 1 To #NUM_PARTICLES
          x = Random(#xmax)+1 : y = Random (#ymax)+1
          StartDrawing(ImageOutput(#Image1))
          While Point(x+1, y+1) + Point(x, y+1)+Point(x+1, y)+Point(x-1, y-1)+Point(x-1, y)+Point(x, y-1) = 0
              x = x + (Random(2)-1) : y = y + (Random(2)-1) 
              If x < 1 Or x > #xmax Or y < 1 Or y > #ymax
                  x = Random(#xmax)+1 : y = Random (#ymax)+1
              EndIf   
          Wend
          Plot(x,y) 
          StopDrawing()
          SetGadgetState(#ImgGadget, ImageID(#Image1))          
      Next
 
   EndIf
 
    Repeat
      Event = WaitWindowEvent()
    Until Event = #PB_Event_CloseWindow
EndIf
; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 10
; EnableUnicode
; EnableXP