﻿;==================================================================
; Rosetta Task: Caesar cipher - Alternate

; Implement a Caesar cipher, both encoding And decoding. 
; The key is an integer from 1 To 25.
; 
; This cipher rotates (either towards left or right) the letters of the alphabet (A To Z).
; 
; The encoding replaces each letter with the 1st to 25th next letter in the alphabet (wrapping Z to A). 
; So key 2 encrypts "HI" to "JK", but key 20 encrypts "HI" to "BC".
; 
; This simple "monoalphabetic substitution cipher" provides almost no security, because an 
; attacker who has the encoded message can either use frequency analysis to guess the key, or just try all 25 keys.
; 
; Caesar cipher is identical to Vigenère cipher with a key of length 1. 
; Also, Rot-13 is identical to Caesar cipher with key 13.

;==================================================================

; Alternate solution... see Ceasar cipher.pb

; Here is an alternate and more advanced form of the encrypt Procedure. It improves on the simple version 
; in terms of speed, in Case Caesar is using the cipher on some very long documents. It is meant to replace 
; the encrypt procedure in the previous code and produces identical results.

Procedure.s CC_encrypt(text.s, key, reverse = 0)
  ;if reverse <> 0 then reverse the encryption (decrypt)
  Protected i, *letter.Character, *resultLetter.Character, result.s = Space(Len(text))
 
  If reverse: key = 26 - key: EndIf
  If key < 1 Or key > 25: ProcedureReturn: EndIf  ;exit if key out of range
 
  *letter = @text: *resultLetter = @result
  While *letter\c
    Select *letter\c
      Case 'A' To 'Z'
        *resultLetter\c = ((*letter\c - 65 + key) % 26) + 65
      Case 'a' To 'z'
        *resultLetter\c = ((*letter\c - 97 + key) % 26) + 97
      Default
        *resultLetter\c = *letter\c
    EndSelect
    *letter + SizeOf(Character): *resultLetter + SizeOf(Character)
  Wend
  ProcedureReturn result
EndProcedure

Procedure.s CC_decrypt(cypherText.s, key)
  ProcedureReturn CC_encrypt(cypherText, key, 1)
EndProcedure
 
If OpenConsole()
  Define key, plainText.s, encryptedText.s, decryptedText.s
 
  key = Random(24) + 1 ;get a random key in the range 1 -> 25
 
  plainText = "The quick brown fox jumped over the lazy dogs.": PrintN(RSet("Plain text = ", 17) + #DQUOTE$ + plainText + #DQUOTE$)
  encryptedText = CC_encrypt(plainText, key): PrintN(RSet("Encrypted text = ", 17) + #DQUOTE$ + encryptedText + #DQUOTE$)
  decryptedText = CC_decrypt(encryptedText, key): PrintN(RSet("Decrypted text = ", 17) + #DQUOTE$ + decryptedText + #DQUOTE$)
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit"): Input()
  CloseConsole()
EndIf
; Output:
;     Plain text = "The quick brown fox jumped over the lazy dogs."
; Encrypted text = "Znk waoiq hxuct lud pasvkj ubkx znk rgfe jumy."
; Decrypted text = "the quick brown fox jumped over the lazy dogs."
; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 23
; Folding = -
; EnableUnicode
; EnableXP