﻿;==================================================================
; Rosetta Task: Color quantization

; full color
; 
; Example: Gimp 16 color
; Color quantization is the process of reducing number of colors used in an image While trying To maintain the 
; visual appearance of the original image. In general, it is a form of cluster analysis, If each RGB color value is
;  considered As a coordinate triple in the 3D colorspace. There are some well know algorithms [1], each 
; With its own advantages And drawbacks.
; 
; Task: Take an RGB color image and reduce its colors to some smaller number (< 256). For this task, use the
;  frog as input and reduce colors to 16, and output the resulting colors. The chosen colors should be adaptive 
;  to the input image, meaning you should not use a fixed palette such as web colors or Windows system 
;  palette. Dithering is not required.
; 
; Note: the funny color bar on top of the frog image is intentional. 

;==================================================================

; ColorQuantization.pb
 
Structure bestA_ ; table for our histogram
nn.i ; 16,32,...
rc.i ; red   count within (0,1,...,255)/(number of colors)
gc.i ; green count within (0,1,...,255)/(number of colors)
bc.i ; blue  count within (0,1,...,255)/(number of colors)
EndStructure
 
; these two functions appear to be rather self-explanatory
UsePNGImageDecoder()
UsePNGImageEncoder()
 
Procedure.i ColorQuantization(Filename$,ncol)
Protected x,y,c
 
; load our original image or leave the procedure
If Not LoadImage(0,Filename$)      :ProcedureReturn 0:EndIf
 
; we are not going to actually draw on the original image...
; but we need to use the drawing library to load up
; the pixel information into our arrays...
; if we can't do that, what's the point of going any further?
; so then we would be wise to just leave the procedure [happy fred?]
If Not StartDrawing(ImageOutput(0)):ProcedureReturn 0:EndIf
 
iw=ImageWidth(0)
ih=ImageHeight(0)
 
Dim cA(iw,ih) ; color array to hold at a given (x,y)
Dim rA(iw,ih) ; red   array to hold at a given (x,y)
Dim gA(iw,ih) ; green array to hold at a given (x,y)
Dim bA(iw,ih) ; blue  array to hold at a given (x,y)
Dim tA(iw,ih) ; temp  array to hold at a given (x,y)
 
; map each pixel from the original image to our arrays
; don't overrun the ranges ie. use {ih-1,iw-1}
For y=0 To ih-1
  For x=0 To iw-1
  c = Point(x,y)
  cA(x,y)=c
  rA(x,y)=Red(c)
  gA(x,y)=Green(c)
  bA(x,y)=Blue(c)
  Next 
Next 
 
StopDrawing() ; don't forget to... StopDrawing()
 
N=ih*iw
; N is the total number if pixels
If Not N:ProcedureReturn 0:EndIf ; to avoid a division by zero
 
; stuctured array ie. a table to hold the frequency distribution
Dim bestA.bestA_(ncol)
 
; the "best" red,green,blue based upon frequency 
Dim rbestA(ncol/3)
Dim gbestA(ncol/3)
Dim bbestA(ncol/3)
 
; split the (0..255) range up
xoff=256/ncol   ;256/16=16
xrng=xoff       ;xrng=16
 
; store these values in our table: bestA(i)\nn= 16,32,...
For i=1 To ncol
xrng+xoff
bestA(i)\nn=xrng
Next 
 
; scan by row [y] 
For y=0 To ih-1
; scan by col [x] 
For x=0 To iw-1
 
; retrieve the rgb values from each pixel 
r=rA(x,y)
g=gA(x,y)
b=bA(x,y)
 
; sum up the numbers that fall within our subdivisions of (0..255)
For i=1 To ncol
If r>=bestA(i)\nn And r<bestA(i+1)\nn:bestA(i)\rc+1:EndIf
If g>=bestA(i)\nn And g<bestA(i+1)\nn:bestA(i)\gc+1:EndIf
If b>=bestA(i)\nn And b<bestA(i+1)\nn:bestA(i)\bc+1:EndIf
Next 
Next 
Next 
 
; option and type to: Sort our Structured Array
opt=#PB_Sort_Descending
typ=#PB_Integer
 
; sort to get most frequent reds
off=OffsetOf(bestA_\rc)
SortStructuredArray(bestA(),opt, off, typ,1, ncol)
 
; save the best [ for number of colors =16 this is int(16/3)=5 ] reds
For i=1 To ncol/3
rbestA(i)=bestA(i)\nn
Next
 
; sort to get most frequent greens
off=OffsetOf(bestA_\gc)
SortStructuredArray(bestA(),opt, off, typ,1, ncol)
 
; save the best [ for number of colors =16 this is int(16/3)=5 ] greens
For i=1 To ncol/3
gbestA(i)=bestA(i)\nn
Next
 
; sort to get most frequent blues
off=OffsetOf(bestA_\bc)
SortStructuredArray(bestA(),opt, off, typ,1, ncol)
 
; save the best [ for number of colors =16 this is int(16/3)=5 ] blues
For i=1 To ncol/3
bbestA(i)=bestA(i)\nn
Next
 
; reset the best low value to 15 and high value to 240
; this helps to ensure there is some contrast when the statistics bunch up
; ie. when a single color tends to predominate... such as perhaps green?
rbestA(1)=15:rbestA(ncol/3)=240
gbestA(1)=15:gbestA(ncol/3)=240
bbestA(1)=15:bbestA(ncol/3)=240
 
; make a copy of our original image or leave the procedure
If Not CopyImage(0,1)              :ProcedureReturn 0:EndIf
 
; draw on that copy of our original image or leave the procedure
If Not StartDrawing(ImageOutput(1)):ProcedureReturn 0:EndIf
 
For y=0 To ih-1
For x=0 To iw-1
c = Point(x,y)
 
; get the rgb value from our arrays
rt=rA(x,y)
gt=gA(x,y)
bt=bA(x,y)
 
; given a particular red value say 123 at point x,y
; which of our rbestA(i's) is closest?
; then for green and blue?
; ==============================
r=255
For i=1 To ncol/3
rdiff=Abs(rbestA(i)-rt)
If rdiff<=r:ri=i:r=rdiff:EndIf
Next
 
g=255
For i=1 To ncol/3
gdiff=Abs(gbestA(i)-gt)
If gdiff<=g:gi=i:g=gdiff:EndIf
Next
 
b=255
For i=1 To ncol/3
bdiff=Abs(bbestA(i)-bt)
If bdiff<=b:bi=i:b=bdiff:EndIf
Next
; ==============================
 
 
; get the color value so we can plot it at that pixel
Color=RGB(rbestA(ri),gbestA(gi),bbestA(bi))
 
; plot it at that pixel
Plot(x,y,Color)
 
; save that info to tA(x,y) for our comparison image 
tA(x,y)=Color
 
Next 
Next 
StopDrawing() ; don't forget to... StopDrawing()
 
; create a comparison image of our original vs 16-color or leave the procedure
If Not CreateImage(2,iw*2,ih)      :ProcedureReturn 0:EndIf
; draw on that image both our original image and our 16-color image or leave the procedure
If Not StartDrawing(ImageOutput(2)):ProcedureReturn 0:EndIf
 
; plot original image
; 0,0 .... 511,0
; .
; .
; 511,0 .. 511,511
For y=0 To ih-1
  For x=0 To iw-1
  c = cA(x,y)
  Plot(x,y,c)
  Next 
  Next 
 
; plot 16-color image to the right of original image
; 512,0 .... 1023,0
; .
; .
; 512,511 .. 1023,511
For y=0 To ih-1
  For x=0 To iw-1
  c = tA(x,y)
  Plot(x+iw,y,c)
  Next 
  Next 
 
StopDrawing() ; don't forget to... StopDrawing()
 
; save the single 16-color image
SaveImage(1, "_single_"+Str(ncol)+"_"+Filename$,#PB_ImagePlugin_PNG )
 
; save the comparison image
SaveImage(2, "_compare_"+Str(ncol)+"_"+Filename$,#PB_ImagePlugin_PNG )
ProcedureReturn 1
EndProcedure
 
ColorQuantization("Quantum_frog.png",16)
; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 13
; FirstLine = 9
; Folding = -
; EnableUnicode
; EnableXP