﻿;==================================================================
; Rosetta Task: Combinations with repetitions

; The set of combinations with repetitions is computed from a set. In the real world, 
; it is about choosing sets where there is a “large” supply of each type of element and where the 
; order of choice does not matter. 
;For example:
; Q: How many ways can a person choose two doughnuts from a store selling three types of 
; doughnut: iced, jam, and plain?
; Note that both the order of items within a pair, and the order of the pairs given in the 
; answer is not significant; the pairs represent multisets. 
; Also note that doughnut can also be spelled donut.
; 
; Task description
; 
; Write a function/program/routine/.. to generate all the combinations with repetitions of {n}  
; types of things taken {k}  at a time and use it to show an answer to the doughnut example above.
; For extra credit, use the function to compute and show just the number of ways of choosing three 
; doughnuts from a choice of ten types of doughnut. Do not show the individual choices for this part.

;==================================================================

Procedure nextCombination(Array combIndex(1), elementCount)
  ;combIndex() must be dimensioned to 'k' - 1, elementCount equals 'n' - 1
  ;combination produced includes repetition of elements and is represented by the array combIndex()
  Protected i, indexValue, combSize = ArraySize(combIndex()), curIndex
 
  ;update indexes
  curIndex = combSize
  Repeat 
    combIndex(curIndex) + 1
    If combIndex(curIndex) > elementCount
 
      curIndex - 1
      If curIndex < 0
        For i = 0 To combSize
          combIndex(i) = 0
        Next 
        ProcedureReturn #False ;array reset to first combination
      EndIf 
 
    ElseIf curIndex < combSize
 
      indexValue = combIndex(curIndex)
      Repeat
        curIndex + 1
        combIndex(curIndex) = indexValue
      Until curIndex = combSize
 
    EndIf
  Until  curIndex = combSize
 
  ProcedureReturn #True ;array contains next combination
EndProcedure
 
Procedure.s display(Array combIndex(1), Array dougnut.s(1))
  Protected i, elementCount = ArraySize(combIndex()), output.s = "  "
  For i = 0 To elementCount
    output + dougnut(combIndex(i)) + " + "
  Next
  ProcedureReturn Left(output, Len(output) - 3)
EndProcedure
 
DataSection
  Data.s "iced", "jam", "plain"
EndDataSection
 
If OpenConsole()
  Define n = 3, k = 2, i, combinationCount
  Dim combIndex(k - 1)
  Dim dougnut.s(n - 1)
  For i = 0 To n - 1: Read.s dougnut(i): Next
 
  PrintN("Combinations of " + Str(k) + " dougnuts taken " + Str(n) + " at a time with repetitions.")
  combinationCount = 0
  Repeat
    PrintN(display(combIndex(), dougnut()))
    combinationCount + 1
  Until Not nextCombination(combIndex(), n - 1)
  PrintN("Total combination count: " + Str(combinationCount))
 
  ;extra credit
  n = 10: k = 3
  Dim combIndex(k - 1)
  combinationCount = 0
  Repeat: combinationCount + 1: Until Not nextCombination(combIndex(), n - 1)
  PrintN(#CRLF$ + "Ways to select " + Str(k) + " items from " + Str(n) + " types: " + Str(combinationCount))
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit"): Input()
  CloseConsole()
EndIf 

;The nextCombination() Procedure operates on an array of indexes to produce the next combination. 
; This generalization allows producing a combination from any collection of elements. nextCombination() 
; returns the value #False when the indexes have reach their maximum values and are then reset.

; Output:
; Combinations of 2 dougnuts taken 3 at a time with repetitions.
;   iced + iced
;   iced + jam
;   iced + plain
;   jam + jam
;   jam + plain
;   plain + plain
; Total combination count: 6
; 
; Ways To Select 3 items from 10 types: 220
; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 18
; Folding = -
; EnableUnicode
; EnableXP