﻿;==================================================================
; Rosetta Task: Conway's Game of Life

; The Game of Life is a cellular automaton devised by the British mathematician John Horton Conway 
; in 1970. It is the best-known example of a cellular automaton.
; 
; Conway's game of life is described here:
; 
; A cell C is represented by a 1 when alive or 0 when dead, in an m-by-m square array of cells. 
; We calculate N - the sum of live cells in C's eight-location neighbourhood, then cell C is alive or 
; dead in the next generation based on the following table:
; 
;    C   N                 new C
;    1   0,1             ->  0  # Lonely
;    1   4,5,6,7,8       ->  0  # Overcrowded
;    1   2,3             ->  1  # Lives
;    0   3               ->  1  # It takes three To give birth!
;    0   0,1,2,4,5,6,7,8 ->  0  # Barren
; Assume cells beyond the boundary are always dead.
; 
; The "game" is actually a zero-player game, meaning that its evolution is determined by its initial 
; state, needing no input from human players. One interacts with the Game of Life by creating an 
; initial configuration and observing how it evolves.
; 
; Although you should test your implementation on more complex examples such as the glider in a 
; larger universe, show the action of the blinker (three adjoining cells in a row all alive), over 
; three generations, in a 3 by 3 grid.

;==================================================================

EnableExplicit
Define.i x, y ,Xmax ,Ymax ,N
Xmax = 13 : Ymax = 20
Dim     world.i(Xmax+1,Ymax+1)
Dim Nextworld.i(Xmax+1,Ymax+1)  
 
; Glider test
;------------------------------------------
 world(1,1)=1 : world(1,2)=0 : world(1,3)=0
 world(2,1)=0 : world(2,2)=1 : world(2,3)=1
 world(3,1)=1 : world(3,2)=1 : world(3,3)=0
;------------------------------------------
 
OpenConsole()
EnableGraphicalConsole(1)
ClearConsole()
Print("Press any key to interrupt")
Repeat
  ConsoleLocate(0,2)
  PrintN(LSet("", Xmax+2, "-"))
 ;---------- endless world ---------
  For y = 1 To Ymax
    world(0,y)=world(Xmax,y)
    world(Xmax+1,y)=world(1,y)  
  Next
  For x = 1 To Xmax
    world(x,0)=world(x,Ymax)
    world(x,Ymax+1)=world(x,1)
  Next
  world(0     ,0     )=world(Xmax,Ymax)
  world(Xmax+1,Ymax+1)=world(1   ,1   )
  world(Xmax+1,0     )=world(1   ,Ymax)
  world(     0,Ymax+1)=world(Xmax,1   )
 ;---------- endless world ---------
  For y = 1 To Ymax
    Print("|") 
    For x = 1 To Xmax
      Print(Chr(32+world(x,y)*3))
      N = world(x-1,y-1)+world(x-1,y)+world(x-1,y+1)+world(x,y-1)
      N + world(x,y+1)+world(x+1,y-1)+world(x+1,y)+world(x+1,y+1)
      If (world(x,y) And (N = 2 Or N = 3))Or (world(x,y)=0 And N = 3)
        Nextworld(x,y)=1      
      Else
        Nextworld(x,y)=0
      EndIf
    Next
    PrintN("|")
  Next
  PrintN(LSet("", Xmax+2, "-"))
  Delay(100) 
  ;Swap world() , Nextworld()    ;PB  <4.50
  CopyArray(Nextworld(), world());PB =>4.50
  Dim Nextworld.i(Xmax+1,Ymax+1) 
Until Inkey() <> ""
 
PrintN("Press any key to exit"): Repeat: Until Inkey() <> ""
; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 8
; FirstLine = 1
; EnableUnicode
; EnableXP