﻿;==================================================================
; Rosetta Task: Deal cards for FreeCell

; Free Cell is the solitaire card game that Paul Alfille introduced to the PLATO system in 1978. Jim Horne, 
; at Microsoft, changed the name to FreeCell and reimplemented the game for DOS, then Windows. 
; This version introduced 32000 numbered deals. (The FreeCell FAQ tells this history.)
; 
; As the game became popular, Jim Horne disclosed the algorithm, and other implementations of FreeCell 
; began to reproduce the Microsoft deals. 
; These deals are numbered from 1 to 32000. Newer versions from Microsoft have 1 million deals, numbered 
; from 1 to 1000000; some implementations allow numbers outside that range.
;
; The algorithm follows:
; 
; Seed the RNG with the number of the deal.
; Create an array of 52 cards: Ace of Clubs, Ace of Diamonds, Ace of Hearts, Ace of Spades, 2 of Clubs, 
; 2 of Diamonds, and so on through the ranks: Ace, 2, 3, 4, 5, 6, 7, 8, 9, 10, Jack, Queen, King. The array 
; indexes are 0 To 51, with Ace of Clubs at 0, and King of Spades at 51.
;
; Until the array is empty:
; Choose a random card at index ≡ next random number (mod Array length).
; Swap this random card with the last card of the array.
; Remove this random card from the array. (array length goes down by 1.)
; Deal this random card.
; Deal all 52 cards, face up, across 8 columns. The first 8 cards go in 8 columns, the next 8 cards go 
; on the first 8 cards, and so on.

; Deals can also be checked against FreeCell solutions to 1000000 games. (Summon a video solution, and 
; it displays the initial deal.)
; 
; Write a program to take a deal number and deal cards in the same order as this algorithm. The program 
; may display the cards with ASCII, with Unicode, by drawing graphics, or any other way.

;==================================================================

#MaxCardNum = 51 ;zero-based count of cards in a deck
Global deckSize
Global Dim cards(#MaxCardNum) ;card with highest index is at the top of deck
 
Procedure RNG(seed.q = -1)
  Static state.q
  If seed >= 0
    state = seed
  Else 
    state = (state * 214013 + 2531011) % (1 << 31) 
    ProcedureReturn state >> 16
  EndIf 
EndProcedure
 
Procedure makeDeck(hand)
  Protected i, c
  For i = 0 To #MaxCardNum: cards(i) = i: Next
 
  RNG(hand) ;set seed value
  deckSize = #MaxCardNum
  While deckSize
    c = RNG() % (deckSize + 1)
    Swap cards(c), cards(deckSize)
    deckSize - 1
  Wend
  deckSize = #MaxCardNum
EndProcedure
 
Procedure showDeck(hand)
  Protected i, c
  PrintN("Hand #" + Str(hand))
  makeDeck(hand)
  For i = 0 To #MaxCardNum
    c = cards(#MaxCardNum - i)
    Print(" " + Mid("A23456789TJQK", (c / 4) + 1, 1) + Mid("CDHS",(c % 4) + 1, 1))
    If (i + 1) % 8 = 0 Or i = #MaxCardNum: PrintN(""): EndIf
  Next 
EndProcedure
 
If OpenConsole()
  showDeck(1)
  showDeck(617)
  showDeck(11982)
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit"): Input()
  CloseConsole()
EndIf
; Sample output:
; Hand #1
;  JD 2D 9H JC 5D 7H 7C 5H
;  KD KC 9S 5S AD QC KH 3H
;  2S KS 9D QD JS As AH 3C
;  4C 5C TS QH 4H AC 4D 7S
;  3S TD 4S TH 8H 2C JH 7D
;  6D 8S 8D QS 6C 3D 8C TC
;  6S 9C 2H 6H
; Hand #617
;  7D AD 5C 3S 5S 8C 2D AH
;  TD 7S QD AC 6D 8H As KH
;  TH QC 3H 9D 6S 8D 3D TC
;  KD 5H 9S 3C 8S 7H 4D JS
;  4C QS 9C 9H 7C 6H 2C 2S
;  4S TS 2H 5D JC 6C JH QH
;  JD KS KC 4H
; Hand #11982
;  AH As 4H AC 2D 6S TS JS
;  3D 3H QS QC 8S 7H AD KS
;  KD 6H 5S 4D 9H JH 9S 3C
;  JC 5D 5C 8C 9D TD KH 7C
;  6C 2C TH QH 6D TC 4S 7S
;  JD 7D 8H 9C 2H QD 4C 5H
;  KC 8D 2S 3S

; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 31
; Folding = -
; EnableXP
; EnableUnicode