﻿;==================================================================
; Rosetta Task: Dinesman multiple-dwelling problem
; 
; The task is to solve Dinesman's multiple dwelling problem but in a way that most naturally follows the 
; problem statement given below.
; 
; Solutions are allowed (but not required) to parse and interpret the problem text, but should remain 
; flexible and should state what changes to the problem text are allowed. Flexibility and ease of 
; expression are valued.
; 
; Examples may be be split into "setup", "problem statement", and "output" sections where the ease 
; and naturalness of stating the problem and getting an answer, as well as the ease and flexibility 
; of modifying the problem are the primary concerns.
; 
; Example output should be shown here, as well as any comments on the examples flexibility.
; 
; The problem
;    Baker, Cooper, Fletcher, Miller, And Smith live on different floors of an apartment house that contains only five floors.
;    Baker does not live on the top floor.
;    Cooper does not live on the bottom floor.
;    Fletcher does not live on either the top or the bottom floor.
;    Miller lives on a higher floor than does Cooper.
;    Smith does not live on a floor adjacent to Fletcher's.
;    Fletcher does not live on a floor adjacent to Cooper's.
;    Where does everyone live?

;==================================================================

Prototype cond(Array t(1))
 
Enumeration #Null
  #Baker
  #Cooper
  #Fletcher
  #Miller
  #Smith 
EndEnumeration
 
Procedure checkTenands(Array tenants(1), Array Condions.cond(1))
  Protected i, j
  Protected.cond *f 
  j=ArraySize(Condions())
  For i=0 To j
    *f=Condions(i)              ; load the function pointer to the current condition
    If *f(tenants()) = #False
      ProcedureReturn  #False
    EndIf 
  Next
  ProcedureReturn #True
EndProcedure
 
Procedure C1(Array t(1))
  If Int(Abs(t(#Fletcher)-t(#Cooper)))<>1
    ProcedureReturn #True
  EndIf 
EndProcedure
 
Procedure C2(Array t(1))
  If t(#Baker)<>5
    ProcedureReturn #True
  EndIf 
EndProcedure
 
Procedure C3(Array t(1))
  If t(#Cooper)<>1
    ProcedureReturn #True
  EndIf 
EndProcedure
 
Procedure C4(Array t(1))
  If t(#Miller) >= t(#Cooper)
    ProcedureReturn #True
  EndIf 
EndProcedure
 
Procedure C5(Array t(1))
  If t(#Fletcher)<>1 And t(#Fletcher)<>5
    ProcedureReturn #True
  EndIf 
EndProcedure
 
Procedure C6(Array t(1))
  If Int(Abs(t(#Smith)-t(#Fletcher)))<>1
    ProcedureReturn #True
  EndIf 
EndProcedure
 
 
If OpenConsole()
  Dim People(4)
  Dim Conditions(5)
  Define a, b, c, d, e, i
  ;
  ;- Load all conditions
  Conditions(i)=@C1(): i+1
  Conditions(i)=@C2(): i+1
  Conditions(i)=@C3(): i+1
  Conditions(i)=@C4(): i+1
  Conditions(i)=@C5(): i+1
  Conditions(i)=@C6()
  ;
  ; generate and the all legal combinations
  For a=1 To 5
    For b=1 To 5
      If a=b: Continue: EndIf
      For c=1 To 5
        If a=c Or b=c: Continue: EndIf
        For d=1 To 5
          If d=a Or d=b Or d=c : Continue: EndIf
          For e=1 To 5 
            If e=a Or e=b Or e=c Or e=d: Continue: EndIf
            People(#Baker)=a
            People(#Cooper)=b
            People(#Fletcher)=c
            People(#Miller)=d
            People(#Smith)=e
            If checkTenands(People(), Conditions())
              PrintN("Solution found;")
              PrintN("Baker="+Str(a)+#CRLF$+"Cooper="+Str(b)+#CRLF$+"Fletcher="+Str(c))
              PrintN("Miller="+Str(d)+#CRLF$+"Smith="+Str(e)+#CRLF$) 
            EndIf
          Next
        Next
      Next
    Next
  Next
  Print("Press ENTER to exit"): Input()
EndIf
; Solution found;
; Baker=3
; Cooper=2
; Fletcher=4
; Miller=5
; Smith=1
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 23
; Folding = --
; EnableXP
; EnableUnicode