﻿;==================================================================
; Rosetta Task: Entropy

; Calculate the information entropy (Shannon entropy) of a given input string.
; 
; Entropy is the expected value of the measure of information content in a system. In general, the Shannon 
; entropy of a variable{X} is defined as:
; 
;  H(X)=\sum _{x\in \Omega }P(x)I(x)}
; where the information content {I(x)=-\log _{b}P(x)} . If the base of the logarithm  
; {b=2} , the result is expressed in bits, a unit of information. Therefore, given a string {S}  
; of length {n}  where {P(s_{i})}  is the relative frequency of each character, 
; the entropy of a string in bits is:
; 
; {H(S)=-\sum _{i=0}^{n}P(s_{i})\log _{2}(P(s_{i}))}

; For this task, use "1223334444" As an example. The result should be around 1.84644 bits.
;==================================================================


#TESTSTR="1223334444"
NewMap uchar.i() : Define.d e
 
Procedure.d nlog2(x.d) : ProcedureReturn Log(x)/Log(2) : EndProcedure
 
Procedure countchar(s$, Map uchar())
  If Len(s$)
    uchar(Left(s$,1))=CountString(s$,Left(s$,1))
    s$=RemoveString(s$,Left(s$,1))
    ProcedureReturn countchar(s$, uchar())
  EndIf
EndProcedure
 
countchar(#TESTSTR,uchar())
 
ForEach uchar()  
  e-uchar()/Len(#TESTSTR)*nlog2(uchar()/Len(#TESTSTR))  
Next
 
OpenConsole()
Print("Entropy of ["+#TESTSTR+"] = "+StrD(e,15))
Input()

; Output:
; Entropy of [1223334444] = 1.846439344671015
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 14
; Folding = -
; EnableXP
; EnableUnicode