﻿;==================================================================
; Rosetta Task: Fibonacci sequence - Recursive optimized calculation
;
; The Fibonacci sequence is a sequence Fn of natural numbers defined recursively:
; F0 = 0
; F1 = 1
; Fn = Fn-1 + Fn-2, If n>1
;
; Write a function to generate the nth Fibonacci number. Solutions can be iterative or recursive 
; (though recursive solutions are generally considered too slow and are mostly used as an exercise in recursion).
; 
; The sequence is sometimes extended into negative numbers by using a 
; straightforward inverse of the positive definition:
; Fn = Fn+2 - Fn+1, If n<0
;
; Support for negative n in the solution is optional.
;==================================================================

; Recursive & optimized With a Static hash table

; This will be much faster on larger n's, this as it uses a table to store known parts instead of recalculating 
; them. On my machine the speedup compares to above code is
; 
; Fib(n) Speedup
; 20           2
; 25          23
; 30         217
; 40       25847
; 46     1156741

Procedure Fibonacci(n)
  Static NewMap Fib.i()
  Protected FirstRecursion
 
  If MapSize(Fib())= 0        ; Init the hash table the first run
    Fib("0")=0: Fib("1")=1
    FirstRecursion = #True
  EndIf
 
  If n >= 2
    Protected.s s=Str(n)
    If Not FindMapElement(Fib(),s)  ; Calculate only needed parts
      Fib(s)= Fibonacci(n-1)+Fibonacci(n-2)
    EndIf
    n = Fib(s)  
  EndIf
  If FirstRecursion ; Free the memory when finalizing the first call
    ClearMap(Fib())
  EndIf
  ProcedureReturn n
EndProcedure

Debug Fibonacci(5)

; Example
; 
; Fibonacci(0)= 0
; Fibonacci(1)= 1
; Fibonacci(2)= 1
; Fibonacci(3)= 2
; Fibonacci(4)= 3
; Fibonacci(5)= 5
; 
; FibonacciReq(0)= 0
; FibonacciReq(1)= 1
; FibonacciReq(2)= 1
; FibonacciReq(3)= 2
; FibonacciReq(4)= 3
; FibonacciReq(5)= 5
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 15
; Folding = -
; EnableXP
; EnableUnicode