﻿;==================================================================
; Rosetta Task: Gray Code

; Gray code is a form of binary encoding where transitions between consecutive numbers differ by only 
; one bit. This is a useful encoding for reducing hardware data hazards with values that change rapidly
; and/or connect to slower hardware as inputs. It is also useful for generating inputs For Karnaugh maps
; in order from left to right or top to bottom.

; Create functions to encode a number to and decode a number from Gray code.
; 
; Display the normal binary representations, Gray code representations, and decoded Gray code values 
; For all 5-bit binary numbers (0-31 inclusive, leading 0's not necessary).
; 
; There are many possible Gray codes. The following encodes what is called "binary reflected Gray code."
;==================================================================

Procedure.i gray_encode(n)
  ProcedureReturn n ! (n >> 1)
EndProcedure
 
Procedure.i gray_decode(g)
  Protected bit = 1 << (8 * SizeOf(Integer) - 2)
  Protected b = g & bit, p = b >> 1
 
  While bit > 1
    bit >> 1
    b | (p ! (g & bit))
    p = (b & bit) >> 1
  Wend 
  ProcedureReturn b
EndProcedure
 
If OpenConsole()
  PrintN("Number Binary Gray    Decoded")
  Define i, n
  For i = 0 To 31
    g = gray_encode(i)
    Print(RSet(Str(i), 2, "0") + Space(5))
    Print(RSet(Bin(g, #PB_Byte), 5, "0") + Space(2))
    n = gray_decode(g)
    Print(RSet(Bin(n, #PB_Byte), 5, "0") + Space(3))
    PrintN(RSet(Str(n), 2, "0"))
  Next
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit"): Input()
  CloseConsole()
EndIf
; --------------------------------------
; Output:
; Number Binary Gray    Decoded
; 00     00000  00000   00
; 01     00001  00001   01
; 02     00011  00010   02
; 03     00010  00011   03
; 04     00110  00100   04
; 05     00111  00101   05
; 06     00101  00110   06
; 07     00100  00111   07
; 08     01100  01000   08
; 09     01101  01001   09
; 10     01111  01010   10
; 11     01110  01011   11
; 12     01010  01100   12
; 13     01011  01101   13
; 14     01001  01110   14
; 15     01000  01111   15
; 16     11000  10000   16
; 17     11001  10001   17
; 18     11011  10010   18
; 19     11010  10011   19
; 20     11110  10100   20
; 21     11111  10101   21
; 22     11101  10110   22
; 23     11100  10111   23
; 24     10100  11000   24
; 25     10101  11001   25
; 26     10111  11010   26
; 27     10110  11011   27
; 28     10010  11100   28
; 29     10011  11101   29
; 30     10001  11110   30
; 31     10000  11111   31
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 10
; Folding = -
; EnableXP
; EnableUnicode