﻿;==================================================================
; Rosetta Task: Grayscale image

; Many image processing algorithms are defined for grayscale (or else monochromatic) images. Extend the 
; data storage type defined on this page to support grayscale images. Define two operations, one to convert
; a color image to a grayscale image and one for the backward conversion. To get luminance of a color 
; use the formula recommended by CIE:
; 
; L = 0.2126·R + 0.7152·G + 0.0722·B
; 
; When using floating-point arithmetic make sure that rounding errors would not cause run-time problems 
; or else distorted results when calculated luminance is stored as an unsigned integer.
; ==================================================================

Procedure ImageGrayout(image)
  Protected w, h, x, y, r, g, b, gray, color
  
  w = ImageWidth(image)
  h = ImageHeight(image)
  StartDrawing(ImageOutput(image))
  For x = 0 To w - 1
    For y = 0 To h - 1
      color = Point(x, y)
      r    = Red(color)
      g    = Green(color)
      b    = Blue(color)
      gray = 0.2126*r + 0.7152*g + 0.0722*b
      Plot(x, y, RGB(gray, gray, gray))
    Next
  Next
  StopDrawing()
EndProcedure

Procedure ImageToColor(image)
  Protected w, h, x, y, v, gray
  
  w = ImageWidth(image)
  h = ImageHeight(image)
  StartDrawing(ImageOutput(image))
  For x = 0 To w - 1
    For y = 0 To h - 1
      gray = Point(x, y)
      v    = Red(gray) ;for gray, each of the color's components is the same 
      ;color = RGB(0.2126*v, 0.7152*v, 0.0722*v)
      Plot(x, y, RGB(v, v, v))
    Next
  Next
  StopDrawing()
EndProcedure

; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 2
; FirstLine = 1
; Folding = -
; EnableXP
; EnableUnicode