﻿;==================================================================
; Rosetta Task: Greyscale bars - Display

; The task is to display a series of vertical greyscale bars (contrast bars) with a sufficient number of 
; bars to span the entire width of the display.
; 
; For the top quarter of the display, the left hand bar should be black, and we then incrementally step 
; through six shades of grey until we have a white bar on the right hand side of the display. 
; (This gives a total of 8 bars)
; 
; For the second quarter down, we start with white and step down through 14 shades of gray, getting 
; darker until we have black on the right hand side of the display. (This gives a total of 16 bars).
; 
; Halfway down the display, we start with black, and produce 32 bars, ending in white, and for the 
; last quarter, we start with white and step through 62 shades of grey, before finally arriving at black 
; in the bottom right hand corner, producing a total of 64 bars for the bottom quarter.
;==================================================================

If Not InitKeyboard(): End: EndIf    ;can't init keyboard
If Not InitSprite(): End: EndIf      ;can't init sprite/screen library
If Not ExamineDesktops(): End: EndIf ;can't retrieve information about desktop
 
Define height.f, width.f, depth
height.f = DesktopHeight(0)
width.f = DesktopWidth(0)
depth = DesktopDepth(0)
 
If OpenScreen(width, height, depth, "Press ENTER to exit")
  Define vsCount, v, h, columns, columnWidth, endColor, shade
  StartDrawing(ScreenOutput())
    vsCount = 4
    For v = 0 To 3
      columns = (v + 1) * 8
      columnWidth = Round(width / columns, #PB_Round_Up)
      endColor = $FFFFFF * (v % 2)     ;alternate between black and white for first and last bar
      Box(0, (height * v) / vsCount, columnWidth, height / vsCount, endColor)
 
      For h = 1 To columns - 2
        If v % 2 = 0
          shade = 256 / columns * (h + 1)
        Else
          shade = 256 / columns * (columns - (h + 1))
        EndIf 
        Box((width * h) / columns, (height * v) / vsCount, columnWidth, height / vsCount, RGB(shade, shade, shade))
      Next
 
      Box((width * (columns - 1)) / columns, (height * v) / vsCount, columnWidth, height / vsCount, $FFFFFF - endColor)
    Next 
  StopDrawing()
  FlipBuffers()
 
  Repeat
    Delay(10)
    ExamineKeyboard()
  Until KeyboardPushed(#PB_Key_Escape) Or KeyboardPushed(#PB_Key_Return)
  CloseScreen()
EndIf
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 15
; EnableXP
; EnableUnicode