﻿;==================================================================
; Rosetta Task: History variables
;
; Storing the history of objects in a program is a common task. Maintaining the history of an object in a 
; program has traditionally required programmers either to write specific code for handling the historical 
; data, or to use a library which supports history logging.
;
; History variables are variables in a programming language which store not only their current value, but 
; also the values they have contained in the past. Some existing languages do provide support for history 
; variables. However these languages typically have many limits and restrictions on use of history variables.
;==================================================================

; integer history variable
 
Structure historyint
    List value.i()
EndStructure
 
Procedure SetInt (*var.historyint, val.i)
    AddElement(*var\value())
 
    If (ListSize(*var\value()) = 1)
        *var\value() = 0
        AddElement(*var\value())
 
    EndIf
 
    *var\value() = val  
EndProcedure
 
Procedure ShowHistory (*var.historyint)
    ForEach *var\value()
        Debug *var\value()
    Next
EndProcedure
 
Procedure UndoInt (*var.historyint)
    If (ListSize(*var\value()) = 1)
        ProcedureReturn
    EndIf
 
    DeleteElement(*var\value())  
EndProcedure
 
;----------------------------------------------
 
Define x.historyint
 
For i = 0 To 3
    setint(x, Random(50)+100 )
Next
 
Debug "x history:"
ShowHistory(x)
 
Debug ""
 
For i = 0 To 3
    UndoInt(x)
    Debug "undo, x = "+Str(x\value())
Next
;  
; Output:
; x history:
; 0
; 131
; 109
; 143
; 108
; 
; undo, x = 143
; undo, x = 109
; undo, x = 131
; undo, x = 0
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 9
; Folding = -
; EnableXP
; EnableUnicode