﻿;==================================================================
; Rosetta Task: Huffman coding
;
; Huffman encoding is a way to assign binary codes to symbols that reduces the overall number of bits used
; to encode a typical string of those symbols.
; 
; For example, if you use letters as symbols and have details of the frequency of occurrence of those letters
; in typical strings, then you could just encode each letter with a fixed number of bits, such as in ASCII codes. 
; You can do better than this by encoding more frequently occurring letters such as e and a, with smaller bit
; strings; and less frequently occurring letters such as q and x with longer bit strings.
; 
; Any string of letters will be encoded as a string of bits that are no-longer of the same length per letter. To 
; successfully decode such as string, the smaller codes assigned to letters such as 'e' cannot occur as a prefix 
; in the larger codes such as that for 'x'.
; 
; If you were to assign a code 01 for 'e' and code 011 for 'x', then if the bits to decode started as 011... then 
; you would not know if you should decode an 'e' or an 'x'.
; The Huffman coding scheme takes each symbol and its weight (or frequency of occurrence), and generates 
; proper encodings for each symbol taking account of the weights of each symbol, so that higher weighted 
; symbols have fewer bits in their encoding. (See the WP article for more information).
; 
; A Huffman encoding can be computed by first creating a tree of nodes:
; Create a leaf node for each symbol and add it to the priority queue.
; While there is more than one node in the queue:
; Remove the node of highest priority (lowest probability) twice to get two nodes.
; Create a new internal node with these two nodes as children and with probability equal to the sum of the two nodes' probabilities.
; Add the new node to the queue.
; The remaining node is the root node and the tree is complete.
; Traverse the constructed binary tree from root to leaves assigning and accumulating a '0' for one branch and a '1' for
;  the other at each node. The accumulated zeros and ones at each leaf constitute a Huffman encoding for those symbols and weights:
; 
; Using the characters and their frequency from the string "this is an example for huffman encoding", create a 
; program to generate a Huffman encoding for each character as a table.
; ==================================================================

OpenConsole()
 
SampleString.s="this is an example for huffman encoding"
datalen=Len(SampleString)
 
Structure ztree
  linked.c
  ischar.c
  char.c
  number.l
  left.l
  right.l
EndStructure

Dim memc.c(datalen)
CopyMemory(@SampleString, @memc(0), datalen * SizeOf(Character))
 
Dim tree.ztree(255)
 
For i=0 To datalen-1
  tree(memc(i))\char=memc(i)
  tree(memc(i))\number+1
  tree(memc(i))\ischar=1
Next
 
SortStructuredArray(tree(),#PB_Sort_Descending,OffsetOf(ztree\number),#PB_Integer)
 
For i=0 To 255
  If tree(i)\number=0
    ReDim tree(i-1)
    Break
  EndIf
Next
 
dimsize=ArraySize(tree())
Repeat
  min1.l=0
  min2.l=0
  For i=0 To dimsize
    If tree(i)\linked=0
      If tree(i)\number<min1 Or min1=0
        min1=tree(i)\number
        hmin1=i
      ElseIf tree(i)\number<min2 Or min2=0
        min2=tree(i)\number
        hmin2=i
      EndIf
    EndIf
  Next
 
  If min1=0 Or min2=0
    Break
  EndIf
 
  dimsize+1
  ReDim tree(dimsize)
  tree(dimsize)\number=tree(hmin1)\number+tree(hmin2)\number
  tree(hmin1)\left=dimsize
  tree(hmin2)\right=dimsize
  tree(hmin1)\linked=1
  tree(hmin2)\linked=1
ForEver
 
i=0
While tree(i)\ischar=1
  str.s=""
  k=i
  ZNEXT:
  If tree(k)\left<>0
    str="0"+str
    k=tree(k)\left
    Goto ZNEXT
  ElseIf tree(k)\right<>0
    str="1"+str
    k=tree(k)\right
    Goto ZNEXT
  EndIf
  PrintN(Chr(tree(i)\char)+" "+str)
  i+1
Wend
Input()
 
CloseConsole()

; Output:
;   110
; n 000
; e 1010
; f 1001
; a 1011
; i 1110
; h 0010
; s 11111
; o 0011
; m 0100
; x 01010
; u 01011
; l 01100
; r 01101
; c 01110
; g 01111
; p 10000
; t 10001
; d 11110
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 11
; EnableXP
; EnableUnicode