﻿;==================================================================
; Rosetta Task: Langtons ant
;
; Langton's ant is a cellular automaton that models an ant sitting on a plane of cells, all of which are white 
; initially, facing in one of four directions. Each cell can either be black or white. The ant moves according 
; to the color of the cell it is currently sitting in, with the following rules:
; - If the cell is black, it changes to white and the ant turns left;
; - If the cell is white, it changes to black and the ant turns right;
; - The Ant then moves forward to the next cell, and repeat from step 1.
;
; This rather simple ruleset leads to an initially chaotic movement pattern, and after about 10000 steps, a 
; cycle appears where the ant moves steadily away from the starting location in a diagonal corridor about 
; 10 pixels wide. Conceptually the ant can then travel infinitely far away.
; 
; For this task, start the ant near the center of a 100 by 100 field of cells, which is about big enough to contain 
; the initial chaotic part of the movement. Follow the movement rules for the ant, terminate when it moves out 
; of the region, and show the cell colors it leaves behind.
;==================================================================

#White = $FFFFFF
#Black = 0
#planeHeight = 100
#planeWidth = 100
#canvasID = 0
#windowID = 0
OpenWindow(#windowID, 0, 0, 150, 150, "Langton's ant", #PB_Window_SystemMenu | #PB_Window_ScreenCentered)
CanvasGadget(#canvasID, 25, 25, #planeWidth, #planeHeight)
StartDrawing(CanvasOutput(#canvasID))
  Box(0, 0, #planeWidth, #planeHeight, #White)
StopDrawing()
 
Define event, quit, ant.POINT, antDirection, antSteps
 
ant\x = #planeHeight / 2
ant\y = #planeWidth / 2
Repeat
  Repeat
    event = WindowEvent()
    If event = #PB_Event_CloseWindow
      quit = 1
      event = 0
    EndIf 
  Until event = 0
 
  StartDrawing(CanvasOutput(#canvasID))
    Select Point(ant\x, ant\y)
      Case #Black
        Plot(ant\x, ant\y, #White)
        antDirection = (antDirection + 1) % 4 ;turn left
      Case #White
        Plot(ant\x, ant\y, #Black)
        antDirection = (antDirection - 1 + 4) % 4 ;turn right
    EndSelect
  StopDrawing()
 
  Select antDirection
    Case 0 ;up
      ant\y - 1
    Case 1 ;left
      ant\x - 1
    Case 2 ;down
      ant\y + 1
    Case 3 ;right
      ant\x + 1
  EndSelect
  antSteps + 1
 
  If ant\x < 0 Or ant\x >= #planeWidth Or ant\y < 0 Or ant\y >= #planeHeight
    MessageRequester("Langton's ant status", "Out of bounds after " + Str(antSteps) + " steps.")
    quit = 1
  EndIf    
 
  Delay(10) ;control animation speed and avoid hogging CPU
Until quit = 1

; Sample output:
; 
; Out of bounds after 11669 steps.
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 16
; EnableXP
; EnableUnicode