﻿;==================================================================
; Rosetta Task: Linear congruential generator
;
; The task is to replicate two historic random number generators. One is the rand() function from BSD libc, 
; and the other is the rand() function from the Microsoft C Runtime (MSCVRT.DLL). Each replica must yield
; the same sequence of integers as the original generator, when starting from the same seed.
;==================================================================

Procedure ms_LCG(seed.q = -1)
  Static state.q
  If seed >= 0
    state = seed
  Else
    state = (state * 214013 + 2531011) % (1 << 31) 
    ProcedureReturn state >> 16
  EndIf 
EndProcedure
 
Procedure.q bsd_LCG(seed.q = -1)
  Static state.q
  If seed >= 0
    state = seed
  Else 
    state = (state * 1103515245 + 12345) % (1 << 31) 
    ProcedureReturn state
  EndIf 
EndProcedure
 
If OpenConsole()
  Define i
  PrintN("BSD (seed = 1)")
  bsd_LCG(1)
  For i = 1 To 5
    PrintN(Str(bsd_LCG()))
  Next
 
  PrintN(#CRLF$ + "MS (seed = 1)")
  ms_LCG(1)
  For i = 1 To 5
    PrintN(Str(ms_LCG()))
  Next
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit"): Input()
  CloseConsole()
EndIf

; Sample output:
; 
; BSD (seed = 1)
; 1103527590
; 377401575
; 662824084
; 1147902781
; 2035015474
; 
; MS (seed = 1)
; 41
; 18467
; 6334
; 26500
; 19169
; IDE Options = PureBasic 5.41 LTS (Windows - x86)
; CursorPosition = 45
; FirstLine = 1
; Folding = -
; EnableUnicode
; EnableXP