﻿;==================================================================
; Rosetta Task: Luhn test of credit card numbers
;
; The Luhn test is used by some credit card companies to distinguish valid credit card numbers from 
; what could be a random selection of digits.
; 
; Those companies using credit card numbers that can be validated by the Luhn test have numbers 
; that pass the following test:
; 
; Reverse the order of the digits in the number.
; Take the first, third, ... and every other odd digit in the reversed digits and sum them to form the partial sum s1
; Taking the second, fourth ... and every other even digit in the reversed digits:
; Multiply each digit by two and sum the digits if the answer is greater than nine to form partial sums for the even digits
; Sum the partial sums of the even digits to form s2
; If s1 + s2 ends in zero then the original number is in the form of a valid credit card number as verified by the Luhn test.
; For example, if the trial number is 49927398716:
;==================================================================

DataSection
  Sample:
  Data.s "49927398716"
  Data.s "49927398717"
  Data.s "1234567812345678"
  Data.s "1234567812345670"
  Data.s ""
EndDataSection
 
Procedure isValid(cardNumber.s)
  Protected i, length, s1, s2, s2a
 
  cardNumber = ReverseString(cardNumber)
  length = Len(cardNumber)
  For i = 1 To length Step 2
    s1 + Val(Mid(cardNumber, i, 1))
  Next 
 
  For i = 2 To length Step 2
    s2a = Val(Mid(cardNumber, i, 1)) * 2
    If s2a < 10
      s2 + s2a
    Else
      s2 + 1 + Val(Right(Str(s2a), 1))
    EndIf 
  Next 
 
  If Right(Str(s1 + s2), 1) = "0"
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf 
EndProcedure
 
 
If OpenConsole()
  Define cardNumber.s
 
  Restore Sample
  Repeat 
    Read.s cardNumber
    If cardNumber <> ""
      Print(cardNumber + " is ")
      If isValid(cardNumber)
        PrintN("valid")
      Else
        PrintN("not valid")
      EndIf
    EndIf
  Until cardNumber = ""
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit")
  Input()
  CloseConsole()
EndIf

; Output:
; 49927398716 is valid
; 49927398717 is Not valid
; 1234567812345678 is Not valid
; 1234567812345670 is valid
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 15
; Folding = -
; EnableXP
; EnableUnicode