﻿;==================================================================
; Rosetta Task: Metered concurrency
;
; The goal of this task is to create a counting semaphore used to control the execution of a set of concurrent 
; units. This task intends to demonstrate coordination of active concurrent units through the use of a passive 
; concurrent unit. The operations for a counting semaphore are acquire, release, and count. Each active 
; concurrent unit should attempt to acquire the counting semaphore before executing its assigned duties. 
; In this case the active concurrent unit should report that it has acquired the semaphore. It should sleep for 
; 2 seconds and then release the semaphore.
;==================================================================

; This launches a few threads in parallel, but restricted by the counter. After a thread has completed it releases
;  the Semaphore and a new thread will be able to start.

#Threads=10
#Parallels=3
Global Semaphore=CreateSemaphore(#Parallels)
 
Procedure Worker(*arg)
  WaitSemaphore(Semaphore)
  Debug "Thread #"+Str(*arg)+" active."
  Delay(Random(2000))
  SignalSemaphore(Semaphore)
EndProcedure
 
; Start a multi-thread based work
Dim thread(#Threads)
For i=0 To #Threads
  thread(i)=CreateThread(@Worker(),i)
Next 
Debug "Launcher done."
 
; Wait for all threads to finish before closing down
For i=0 To #Threads
  If IsThread(i)
    WaitThread(i)
  EndIf
Next
; Sample output
; 
; Thread #0 active.
; Thread #2 active.
; Thread #4 active.
; Launcher done.
; Thread #1 active.
; Thread #3 active.
; Thread #5 active.
; Thread #7 active.
; Thread #9 active.
; Thread #6 active.
; Thread #8 active.
; Thread #10 active.
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 12
; Folding = -
; EnableXP
; EnableUnicode