﻿;==================================================================
; Rosetta Task: Mutex
;
; 
; A mutex (abbreviated Mutually Exclusive access) is a synchronization object, a variant of semaphore 
; with k=1. A mutex is said to be seized by a task decreasing k. It is released when the task restores k. 
; Mutexes are typically used to protect a shared resource from concurrent access. A task seizes 
; (or acquires) the mutex, then accesses the resource, and after that releases the mutex.
; 
; A mutex is a low-level synchronization primitive exposed to deadlocking. A deadlock can occur with just
; two tasks and two mutexes (if each task attempts to acquire both mutexes, but in the opposite order). 
; Entering the deadlock is usually aggravated by a race condition state, which leads to sporadic hangups, 
; which are very difficult to track down.
;==================================================================
; 
; PureBasic has the following Mutex functions;
; 
; MyMutex=CreateMutex()
; Result = TryLockMutex(MyMutex)
; LockMutex(MyMutex)
; UnlockMutex(MyMutex)
; FreeMutex(MyMutex)

; Example:

Declare ThreadedTask(*MyArgument)
Define Mutex
 
If OpenConsole()
  Define thread1, thread2, thread3
 
  Mutex = CreateMutex()
  thread1 = CreateThread(@ThreadedTask(), 1):  Delay(5)
  thread2 = CreateThread(@ThreadedTask(), 2):  Delay(5)
  thread3 = CreateThread(@ThreadedTask(), 3)
  WaitThread(thread1)
  WaitThread(thread2)
  WaitThread(thread3)
 
  PrintN(#CRLF$+"Press ENTER to exit"): Input()
  FreeMutex(Mutex)
  CloseConsole()
EndIf
 
Procedure ThreadedTask(*MyArgument)     
  Shared Mutex
  Protected a, b
  For a = 1 To 3     
    LockMutex(Mutex) 
    ; Without Lock-/UnLockMutex() here the output from the parallel threads would be all mixed.
    ; Reading/Writing to shared memory resources are a common use for Mutextes i PureBasic
    PrintN("Thread "+Str(*MyArgument)+": Print 3 numbers in a row:")
    For b = 1 To 3
      Delay(75)
      PrintN("Thread "+Str(*MyArgument)+" : "+Str(b))
    Next          
    UnlockMutex(Mutex) 
  Next    
EndProcedure
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 12
; Folding = -
; EnableXP
; EnableUnicode