﻿;==================================================================
; Rosetta Task: Permutations
;
; Write a program that generates all permutations of n different objects. (Practically numerals!)
;==================================================================

; The procedure nextPermutation() takes an array of integers as input and transforms its contents into the 
; next lexicographic permutation of it's elements (i.e. integers). It returns #True if this is possible. It returns
; #False if there are no more lexicographic permutations left and arranges the elements into the lowest 
; lexicographic permutation. It also returns #False if there is less than 2 elemetns to permute.
; 
; The integer elements could be the addresses of objects that are pointed at instead. In this Case the addresses 
; will be permuted without respect to what they are pointing to (i.e. strings or structures) and the lexicographic 
; order will be that of the addresses themselves.

Macro reverse(firstIndex, lastIndex)
  first = firstIndex
  last = lastIndex
  While first < last
    Swap cur(first), cur(last)
    first + 1
    last - 1
  Wend 
EndMacro
 
Procedure nextPermutation(Array cur(1))
  Protected first, last, elementCount = ArraySize(cur())
  If elementCount < 1
    ProcedureReturn #False ;nothing to permute
  EndIf 
 
  ;Find the lowest position pos such that [pos] < [pos+1]
  Protected pos = elementCount - 1
  While cur(pos) >= cur(pos + 1)
    pos - 1
    If pos < 0
      reverse(0, elementCount)
      ProcedureReturn #False ;no higher lexicographic permutations left, return lowest one instead
    EndIf 
  Wend
 
  ;Swap [pos] with the highest positional value that is larger than [pos]
  last = elementCount
  While cur(last) <= cur(pos)
    last - 1
  Wend
  Swap cur(pos), cur(last)
 
  ;Reverse the order of the elements in the higher positions
  reverse(pos + 1, elementCount)
  ProcedureReturn #True ;next lexicographic permutation found
EndProcedure
 
Procedure display(Array a(1))
  Protected i, fin = ArraySize(a())
  For i = 0 To fin
    Print(Str(a(i)))
    If i = fin: Continue: EndIf
    Print(", ")
  Next
  PrintN("")
EndProcedure
 
If OpenConsole()
  Dim a(2)
  a(0) = 1: a(1) = 2: a(2) =  3
  display(a())
  While nextPermutation(a()): display(a()): Wend
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit"): Input()
  CloseConsole()
EndIf
; Output:
; 1, 2, 3
; 1, 3, 2
; 2, 1, 3
; 2, 3, 1
; 3, 1, 2
; 3, 2, 1

; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 12
; Folding = -
; Markers = 73
; EnableXP
; EnableUnicode