﻿;==================================================================
; Rosetta Task: Pointers and references - pointers
;
; In this task, the goal is to demonstrate common operations on pointers and references.
;==================================================================

; Pointers
; A pointer's name is preceded by an '*' and are variables of type integer that can hold an address 
; (either 32-bit or 64-bit depending on the OS compiled for), and do not differ from other integer variables 
; in this respect. Pointers can be declared as pointing to structured memory and allow that memory to be 
; de-referenced to allow easier manipulation of its contents. The following code creates a pointer to an integer variable:

Define varA.i = 5, varB.i = 0, *myInteger.Integer
 
*myInteger = @varA      ;set pointer to address of an integer variable
varB = *myInteger\i + 3 ;set variable to the 3 + value of dereferenced pointer, i.e varB = 8
; Change pointer to refer to another integer variable:

Define varC.i = 12
*myInteger = @varC
; Change pointer To Not point To anything:

*myInteger = #Null ;or anything evaluating to zero
; Get a pointer to the first element of an Array, or any desired element:

Dim myArray(10)
*myInteger = myArray()
;Or alternatively:
*myInteger = @myArray(0)
;any specific element
*myInteger = @myArray(4) ;element 4
; PureBasic does not provide pointer arithmetic but it does allow integer math operations on its pointers. This makes it easy to implement a variety of pointer arithmetic with the help of the compiler function SizeOf().

*myInteger + 3 * SizeOf(Integer) ;pointer now points to myArray(3)
*myInteger - 2 * SizeOf(Integer) ;pointer now points to myArray(1)
; In a similar manner the compiler function OffsetOf() may also be used to obtain the memory offset for an sub-element of a structure if needed:

Structure employee
  id.i
  name.s
  jobs.s[20] ;array of job descriptions
EndStructure
 
Dim employees.employee(10) ;an array of employee's
 
;set a string pointer to the 6th job of the 4th employee
*myString.String = @employees(3) + OffsetOf(employee\jobs) + 5 * SizeOf(String)

;==================================================================
; Addresses of variables or Procedure
; Addresses to variables and functions are obtained with the '@' operator.
; examples...
; *pointer = @varA
; *pointer = @myFunction()
;
;==================================================================

; Addresses of labels
; To find the address of a label, you put a question mark (?) in front of the label name.

; Getting the address of a lable in the code
text$="'Lab' is at address "+Str(?lab)
MessageRequester("Info",text$)
 
; Using lables to calculate size
text$="Size of the datasetion is "+Str(?lab2-?lab)+" bytes."
MessageRequester("Info",text$)
 
; Using above to copy specific datas
Define individes=(?lab2-?lab1)/SizeOf(Integer)
Dim Stuff(individes-1)  ; As PureBasic uses 0-based arrays
CopyMemory(?lab1,@Stuff(),?lab2-?lab1)
 
DataSection
  lab:
  Data.s "Foo", "Fuu"
  lab1:
  Data.i  3,1,4,5,9,2,1,6
  lab2:
EndDataSection
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 50
; FirstLine = 33
; EnableXP
; EnableUnicode