﻿;==================================================================
; Rosetta Task: Range expansion
;
; A format for expressing an ordered list of integers is to use a comma separated list of either
;  - Individual integers
;  - Or a range of integers denoted by the starting integer separated from the end integer in the range by a dash, '-'. 
;    (The range includes all integers in the interval including both endpoints)
;
; The range syntax is to be used only for, and for every range that expands to more than two values.
; Example... the list of integers:  -6, -3, -2, -1, 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 14, 15, 17, 18, 19, 20
;
; Is accurately expressed by the range expression:  -6,-3-1,3-5,7-11,14,15,17-20 (And vice-versa).
; 
; The task: Expand the range description:  -6,-3--1,3-5,7-11,14,15,17-20
;
; Note that the second element above, is the range from minus 3 to minus 1.
;==================================================================

Procedure rangeexpand(txt.s, List outputList())
  Protected rangesCount = CountString(txt, ",") + 1
  Protected subTxt.s, r, rangeMarker, rangeStart, rangeFinish, rangeIncrement, i
 
  LastElement(outputList())
  For r = 1 To rangesCount
    subTxt = StringField(txt, r, ",")
    rangeMarker = FindString(subTxt, "-", 2)
    If rangeMarker
      rangeStart = Val(Mid(subTxt, 1, rangeMarker - 1))
      rangeFinish = Val(Mid(subTxt, rangeMarker + 1))
 
      If rangeStart > rangeFinish
        rangeIncrement = -1
      Else
        rangeIncrement = 1
      EndIf 
 
      i = rangeStart - rangeIncrement
      Repeat 
        i + rangeIncrement
        AddElement(outputList()): outputList() = i
      Until i = rangeFinish
    Else
      AddElement(outputList()): outputList() = Val(subTxt)
    EndIf 
  Next
EndProcedure 
 
Procedure outputListValues(List values())
  Print("[ ")
  ForEach values()
    Print(Str(values()) + " ") 
  Next
  PrintN("]")
EndProcedure
 
If OpenConsole()
  NewList values()
  rangeexpand("-6,-3--1,3-5,7-11,14,15,17-20", values())
  outputListValues(values())
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit")
  Input()
  CloseConsole()
EndIf
; Output:
; [ -6 -3 -2 -1 3 4 5 7 8 9 10 11 14 15 17 18 19 20 ]
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 15
; Folding = -
; EnableXP
; EnableUnicode