﻿;==================================================================
; Rosetta Task: Rate counter
;
; Counting the frequency at which something occurs is a common activity in measuring performance and 
; managing resources. In this task, we assume that there is some job which we want to perform repeatedly,
;  and we want to know how quickly these jobs are being performed.
; Of interest is the code that performs the actual measurements. Any other code (such as job implementation 
; or dispatching) that is required to demonstrate the rate tracking is helpful, but not the focus.
; 
; Multiple approaches are allowed (even preferable), so long as they can accomplish these goals:
; 
; Run N seconds worth of jobs and/or Y jobs.
; Report at least three distinct times.
; Be aware of the precision and accuracy limitations of your timing mechanisms, and document them if you can.
;==================================================================
; NOTE - 2 methods

; a) - Counting frequence of an event
Procedure.d TimesPSec(Reset=#False)
  Static starttime, cnt
  Protected Result.d, dt
  If Reset
    starttime=ElapsedMilliseconds(): cnt=0
  Else
    cnt+1
    dt=(ElapsedMilliseconds()-starttime)
    If dt
      Result=cnt/(ElapsedMilliseconds()-starttime)
    EndIf
  EndIf
  ProcedureReturn Result*1000
EndProcedure
 
If OpenWindow(0,#PB_Ignore,#PB_Ignore,220,110,"",#PB_Window_SystemMenu)
  Define Event, r.d, GadgetNumber
  ButtonGadget(0,10, 5,200,35,"Click me!")
  ButtonGadget(1,10,70,100,35,"Reset")
  TextGadget  (2,10,45,200,25,"")
  TimesPSec(1)
  Repeat
    Event=WaitWindowEvent()
    If Event=#PB_Event_Gadget 
      GadgetNumber =EventGadget()
      If GadgetNumber=0
        r=TimesPSec()
        SetGadgetText(2,"You are clicking at "+StrD(r,5)+" Hz.")
      ElseIf GadgetNumber=1
        TimesPSec(1)
        SetGadgetText(2,"Counter zeroed.")
      EndIf
    EndIf
  Until Event=#PB_Event_CloseWindow
EndIf
; ----------------------------------------------------------------

; b) - Counting events For a time period (starts when Window is closed)

Procedure DummyThread(arg)
  Define.d dummy=#PI*Pow(arg,2)/4
EndProcedure
 
start=ElapsedMilliseconds()
Repeat
  T=CreateThread(@DummyThread(),Random(100))
  WaitThread(T)
  cnt+1
Until start+10000<=ElapsedMilliseconds(); Count for 10 sec
 
msg$="We got "+Str(cnt)+" st."+Chr(10)+StrF(cnt/10,2)+" threads per sec."
MessageRequester("Counting threads in 10 sec",msg$)

; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 13
; Folding = -
; EnableXP
; EnableUnicode