﻿;==================================================================
; Rosetta Task: Read a specific line from a file
;
; Some languages have special semantics for obtaining a known line number from a file. The task is to 
; demonstrate how to obtain the contents of a specific line within a file. For the purpose of this task 
; demonstrate how the contents of the seventh line of a file can be obtained, and store it in a variable 
; or in memory (for potential future use within the program if the code were to become embedded). 

; If the file does not contain seven lines, or the seventh line is empty, or too big to be retrieved, output 
; an appropriate message. If no special semantics are available for obtaining the required line, it is 
; permissible to read line by line. Note that empty lines are considered and should still be counted. 
; Note that for functional languages or languages without variables or storage, it is permissible to output
; the extracted data to standard output.
;==================================================================

Structure lineLastRead
  lineRead.i
  line.s
EndStructure
 
Procedure readNthLine(file, n, *results.lineLastRead)
  *results\lineRead = 0
  While *results\lineRead < n And Not Eof(file)
    *results\line = ReadString(file)
    *results\lineRead + 1
  Wend
 
  If *results\lineRead = n
    ProcedureReturn 1
  EndIf 
EndProcedure
 
Define filename.s = OpenFileRequester("Choose file to read a line from", "*.*", "All files (*.*)|*.*", 0)
If filename
  Define file = ReadFile(#PB_Any, filename)
  If file
    Define fileReadResults.lineLastRead, lineToRead = 7 ; change to read a different line number <<<<<
    If readNthLine(file, lineToRead, fileReadResults)
      MessageRequester("Results", fileReadResults\line)
    Else
      MessageRequester("Error", "There are less than " + Str(lineToRead) + " lines in file.")
    EndIf 
    CloseFile(file)
  Else
    MessageRequester("Error", "Couldn't open file " + filename + ".")
  EndIf
EndIf 
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 7
; Folding = -
; EnableXP
; EnableUnicode