﻿;==================================================================
; Rosetta Task: Read entire file
;
; Load the entire contents of some text file as a single string variable.
; 
; If applicable, discuss: encoding selection, the possibility of memory-mapping.
; 
; Of course, one should avoid reading an entire file at once if the file is large and the task can be accomplished
; incrementally instead (in which case check File IO); this is for those cases where having the entire file 
; is actually what is wanted.
;==================================================================

; A file can be read with any of the built in commands

;      Number.b = ReadByte(#File)
;      Length.i = ReadData(#File, *MemoryBuffer, LengthToRead)
;      Number.c = ReadCharacter(#File)
;      Number.d = ReadDouble(#File)
;      Number.f = ReadFloat(#File)
;      Number.i = ReadInteger(#File)
;      Number.l = ReadLong(#File)
;      Number.q = ReadQuad(#File)
;      Text$    = ReadString(#File [, Flags])
;      Number.w = ReadWord(#File)

; If the file is a pure text file (no CR/LF etc.), this will work and will read each line until EOL is found.

;      If ReadFile(0, "RC.txt")       
;        Variable$=ReadString(0)     
;        CloseFile(0) 
;      EndIf

; Since PureBasic terminates strings with a #Null and also split the ReadString() is encountering new line chars, 
; any file containing these must be treated as a data stream.

Title$="Select a file"
Pattern$="Text (.txt)|*.txt|All files (*.*)|*.*"
fileName$ = OpenFileRequester(Title$,"",Pattern$,0)
If fileName$
  If ReadFile(0, fileName$)
    length = Lof(0)     
    *MemoryID = AllocateMemory(length)  
    If *MemoryID
      bytes = ReadData(0, *MemoryID, length)
      MessageRequester("Info",Str(bytes)+" bytes were read")
    EndIf
    CloseFile(0)
  EndIf
EndIf
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 33
; FirstLine = 1
; EnableXP
; EnableUnicode