﻿;==================================================================
; Rosetta Task: Rot-13
;
; Implement a "rot-13" function (or procedure, class, subroutine, or other "callable" object).

; Optionally wrap this function in a utility program (like tr, which acts like a common UNIX utility, performing 
; a line-by-line rot-13 encoding of every line of input contained in each file listed on its command line, or 
; (if no filenames are passed thereon) acting as a filter on itsc "standard input."
; 
; (A number of UNIX scripting languages and utilities, such as awk and sed either default to processing files 
; in this way or have command line switches or modules to easily implement these wrapper semantics, e.g., Perl and Python).
; 
; The "rot-13" encoding is commonly known from the early days of Usenet "Netnews" as a way of obfuscating 
; text to prevent casual reading of spoiler or potentially offensive material.
; 
; Many news reader and mail user agent programs have built-in "rot-13" encoder/decoders or have the ability 
; to feed a message through any external utility script for performing this (or other) actions.
; 
; The definition of the rot-13 function is to simply replace every letter of the ASCII alphabet with the letter 
; which is "rotated" 13 characters "around" the 26 letter alphabet from its normal cardinal position 
; (wrapping around from "z" to "a" as necessary).
; 
; Thus the letters "abc" become "nop" and so on. Technically rot-13 is a "monoalphabetic substitution cipher" 
; with a trivial "key".
; 
; A proper implementation should work on upper and lower case letters, preserve case, and pass all
; non-alphabetic characters in the input stream through without alteration.
;==================================================================

Declare.s Rot13(text_to_code.s)
 
If OpenConsole()
  Define txt$
 
  Print("Enter a string to encode: "): txt$=Input()
 
  PrintN("Coded  : "+Rot13(txt$))
  PrintN("Decoded: "+Rot13(Rot13(txt$)))
 
  Print("Press ENTER to quit."): Input()
  CloseConsole()
EndIf
 
Procedure.s Rot13(s.s)
  Protected.i i
  Protected.s t, u
  For i=1 To Len(s)
    t=Mid(s,i,1)
    Select Asc(t)
      Case Asc("a") To Asc("m"), Asc("A") To Asc("M")
        t=Chr(Asc(t)+13)
      Case Asc("n") To Asc("z"), Asc("N") To Asc("Z")
        t=Chr(Asc(t)-13)
    EndSelect
    u+t
  Next  
  ProcedureReturn u
EndProcedure
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 25
; Folding = -
; EnableXP
; EnableUnicode