﻿;==================================================================
; Rosetta Task: Secure temporary file
;
; Create a temporary file, securely and exclusively (opening it such that there are no possible race conditions). 
; It's fine assuming local filesystem semantics (NFS or other networking filesystems can have signficantly 
; more complicated semantics for satisfying the "no race conditions" criteria). 
; The function should... 
; automatically resolve name collisions and should only fail in cases where permission is denied, the 
; filesystem is read-only or full, or similar conditions exist (returning an error or raising an exception 
; as appropriate to the language/environment).
;==================================================================

Procedure.s TempFile()
  Protected a, Result$
 
  For a = 0 To 9999
    Result$ = GetTemporaryDirectory() + StringField(GetFilePart(ProgramFilename()),1,".")
    Result$ + "_" + Str(ElapsedMilliseconds()) + "_(" + RSet(Str(a),4,"0") + ").tmp"
    If FileSize(Result$) = -1                                      ;  -1 = File not found
         Debug Result$
      ProcedureReturn Result$
    EndIf
  Next
 
  ProcedureReturn ""
EndProcedure
 
 
Define File, File$
 
File$ = TempFile()
If File$ <> ""
  File = CreateFile(#PB_Any, File$)
  If File <> 0
    WriteString(File, "Some temporary data here...")
    CloseFile(File)
  EndIf
EndIf
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 6
; Folding = -
; EnableXP
; EnableUnicode