﻿;==================================================================
; Rosetta Task: Sorting algorithms - Bogosort
;
; Bogosort a list of numbers.
; Bogosort simply shuffles a collection randomly until it is sorted.

; "Bogosort" is a perversely inefficient algorithm only used as an in-joke. Its average run-time is O(n!) 
; because the chance that any given shuffle of a set will end up in sorted order is about one in n 
; factorial, and the worst case is infinite since there's no guarantee that a random shuffling will ever
; produce a sorted sequence. Its best case is O(n) since a single pass through the elements may suffice to order them.
;==================================================================
; 

DisableDebugger

Procedure KnuthShuffle (Array a(1))
  Protected i, Size = ArraySize(a())
  For i = 0 To Size             
    Swap a(i), a(Random(Size)) 
  Next 
EndProcedure
 
Procedure isSorted(Array a(1))
  Protected i, Size = ArraySize(a())
  For i = 1 To Size
    If a(i) < a(i - 1)
      ProcedureReturn #False
    EndIf
  Next
  ProcedureReturn #True
EndProcedure
 
Procedure BogoSort(Array a(1))
  Protected Size = ArraySize(a()) + 1, iter
 
  While Not isSorted(a())
    iter + 1
    KnuthShuffle(a())
  Wend
  MessageRequester("Results","Array of " + Str(Size) + " integers required " + Str(iter) + " shuffles To SORT.")
EndProcedure
 
Dim b(10)
For i = 0 To 10
  b(i) = Random(100)
Next


BogoSort(b())
; Output:
; Array of 10 integers required 2766901 shuffles To SORT.
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 8
; Folding = -
; EnableXP
; EnableUnicode