﻿;==================================================================
; Rosetta Task: Sorting algorithms - Strand sort
;
; Implement the Strand sort. This is a way of sorting numbers by extracting shorter sequences of 
; already sorted numbers from an unsorted list.
;==================================================================

Procedure strandSort(List a())
  Protected NewList subList()
  Protected NewList results()
 
  While ListSize(a()) > 0
    ClearList(subList())
    AddElement(subList())
    FirstElement(a())
    subList() = a()
    DeleteElement(a())
    ForEach a()
      If a() >= subList()
        AddElement(subList())
        subList() = a()
        DeleteElement(a())
      EndIf
    Next
 
    ;merge lists
    FirstElement(subList())
    If Not FirstElement(results())
      ;copy all of sublist() to results()
      MergeLists(subList(), results(), #PB_List_Last)
    Else
      Repeat
        If subList() < results()
          InsertElement(results())
          results() = subList()
          DeleteElement(subList())
          If Not NextElement(subList())
            Break
          EndIf
        ElseIf Not NextElement(results())
          ;add remainder of sublist() to end of results()
          MergeLists(subList(), results(), #PB_List_Last)
          Break 
        EndIf
      ForEver
    EndIf 
 
  Wend 
  CopyList(results(), a())
EndProcedure
 
Procedure.s listContents(List a())
  Protected output.s
  PushListPosition(a())
  ForEach a()
    output + Str(a()) + ","
  Next
  PopListPosition(a())
  ProcedureReturn Left(output, Len(output) - 1)
EndProcedure
 
Procedure setupList(List a())
  ClearList(a())
  Protected elementCount, i
 
  elementCount = Random(5) + 10
  For i = 1 To elementCount
    AddElement(a())
    a() = Random(10) - 5
  Next
EndProcedure
 
 
If OpenConsole()
  NewList sample()
  Define i
 
  For i = 1 To 3
    setupList(sample())
    PrintN("List " + Str(i) + ":")
    PrintN("  Before:  " + listContents(sample()))
    strandSort(sample())
    PrintN("  After :  " + listContents(sample()))
    PrintN("")
  Next
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit"): Input()
  CloseConsole()
EndIf
; Sample output:
; 
; List 1:
;   Before:  3,-2,-4,4,-1,-3,-2,-2,2,2,0
;   After :  -4,-3,-2,-2,-2,-1,0,2,2,3,4
; 
; List 2:
;   Before:  -4,4,3,-2,3,-2,5,0,-1,0,5,1
;   After :  -4,-2,-2,-1,0,0,1,3,3,4,5,5
; 
; List 3:
;   Before:  -2,0,-2,5,5,3,-1,-3,5,5,0,2,-4,4,2
;   After :  -4,-3,-2,-2,-1,0,0,2,2,3,4,5,5,5,5
; IDE Options = PureBasic 5.42 Beta 1 LTS (Windows - x86)
; CursorPosition = 101
; Folding = -
; EnableUnicode
; EnableXP