﻿;==================================================================
; Rosetta Task: Stack
;
; A stack is a container of elements with last in, first out access policy. Sometimes it also called LIFO. 
; The stack is accessed through its top. 
; The basic stack operations are:
; - push stores a new element onto the stack top;
; - pop returns the last pushed stack element, while removing it from the stack;
; - empty tests if the stack contains no elements.
;
; Sometimes the last pushed stack element is made accessible for immutable access (for read) 
; or mutable access (for write): 
; - top (sometimes called peek to keep with the p theme) returns the topmost element without modifying the stack.
;
; Stacks allow a very simple hardware implementation. They are common in almost all processors. In programming 
; stacks are also very popular for their way (LIFO) of resource management, usually memory. Nested 
; scopes of language objects are naturally implemented by a stack (sometimes by multiple stacks). 
; This is a classical way to implement local variables of a reentrant or recursive subprogram. Stacks are
; also used to describe a formal computational framework. See stack machine. Many algorithms in pattern
; matching, compiler construction (e.g. recursive descent parsers), and machine learning (e.g. based on tree traversal) 
; have a natural representation in terms of stacks.
; 
; Create a stack supporting the basic operations: push, pop, empty.
;==================================================================

; For LIFO function PureBasic normally uses linked lists. 
; Usage as described above could look like...

Global NewList MyStack()
 
Procedure Push_LIFO(n)
  FirstElement(MyStack())
  InsertElement(MyStack())
  MyStack() = n
EndProcedure
 
Procedure Pop_LIFO()
  If FirstElement(MyStack())
    Topmost = MyStack()
    DeleteElement(MyStack())
  EndIf
  ProcedureReturn Topmost
EndProcedure
 
Procedure Empty_LIFO()
  Protected Result
  If ListSize(MyStack())=0
    Result = #True
  EndIf
  ProcedureReturn Result
EndProcedure
 
Procedure Peek_LIFO()
  If FirstElement(MyStack())
    Topmost = MyStack()
  EndIf
  ProcedureReturn Topmost
EndProcedure
 
;----   Example of implementation ----
Push_LIFO(3)
Push_LIFO(1)
Push_LIFO(4)
While Not Empty_LIFO()
  Debug Pop_LIFO()
Wend
; Output:
;  4
;  1
;  3
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 26
; Folding = -
; EnableXP
; EnableUnicode