﻿;==================================================================
; Rosetta Task: Strip control codes and extended characters from a string
;
; The task is to strip control codes and extended characters from a string. The solution should demonstrate
;  how to achieve each of the following results:
; 
; a string with control codes stripped (but extended characters not stripped)
; a string with control codes and extended characters stripped
; In ASCII, the control codes have decimal codes 0 through to 31 and 127. On an ASCII based system, if the
; control codes are stripped, the resultant string would have all of its characters within the range of 32 to 
; 126 decimal on the ascii table.
; 
; On a non-ASCII based system, we consider characters that do not have a corresponding glyph on the 
; ASCII table (within the ASCII range of 32 to 126 decimal) to be an extended character for the purpose of this task.
;==================================================================

Procedure.s stripControlCodes(source.s)
  Protected i, *ptrChar.Character, length = Len(source), result.s
  *ptrChar = @source
  For i = 1 To length
    If *ptrChar\c > 31 
      result + Chr(*ptrChar\c)
    EndIf
    *ptrChar + SizeOf(Character)
  Next
  ProcedureReturn result 
EndProcedure
 
Procedure.s stripControlExtCodes(source.s)
  Protected i, *ptrChar.Character, length = Len(source), result.s
  *ptrChar = @source
  For i = 1 To length
    If *ptrChar\c > 31 And *ptrChar\c < 128
      result + Chr(*ptrChar\c)
    EndIf
    *ptrChar + SizeOf(Character)
  Next
  ProcedureReturn result 
EndProcedure
 
If OpenConsole()
  ;create sample string
  Define i, s.s
  For i = 1 To 80
    s + Chr(Random(254) + 1) ;include character values from 1 to 255
  Next 
 
  PrintN(stripControlCodes(s))    ;string without control codes 
  PrintN("---------")
  PrintN(stripControlExtCodes(s)) ;string without control codes or extended chars
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit"): Input()
  CloseConsole()
EndIf

; Sample output:
; 
; »╫=┐C─≡G(═ç╤â√╝÷╔¬ÿ▌x  è4∞|)ï└⌐ƒ9²òτ┌ºáj)▓<~-vPÿφQ╨ù¿╖îFh"[ü╗dÉ₧q#óé├p╫■
; ---------
; =CG(x 4|)9j)<~-vPQFh"[dq#p
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 13
; Folding = -
; EnableXP
; EnableUnicode