﻿;==================================================================
; Rosetta Task: Strip whitespace from a string - Top and tail
;
; The task is to demonstrate how to strip leading and trailing whitespace from a string. The solution should 
; demonstrate how to achieve the following three results:
; 
; - String with leading whitespace removed
; - String with trailing whitespace removed
; - String with both leading and trailing whitespace removed
;
; For the purposes of this task whitespace includes non printable characters such as the space character, 
; the tab character, and other such characters that have no corresponding graphical representation.
;==================================================================

; Note, if only spaces need to be removed, PureBasic provides commands that do this: LTrim(), RTrim(), 
; and Trim(). To handle a larger selection of whitespace the following functions meet the task.

;define the whitespace as desired
#whitespace$ = " " + Chr($9) + Chr($A) + Chr($B) + Chr($C) + Chr($D) + Chr($1C) + Chr($1D) + Chr($1E) + Chr($1F)
 
Procedure.s myLTrim(source.s)
  Protected i, *ptrChar.Character, length = Len(source)
  *ptrChar = @source
  For i = 1 To length
    If Not FindString(#whitespace$, Chr(*ptrChar\c))
      ProcedureReturn Right(source, length + 1 - i)
    EndIf
    *ptrChar + SizeOf(Character)
  Next
EndProcedure
 
Procedure.s myRTrim(source.s)
  Protected i, *ptrChar.Character, length = Len(source)
  *ptrChar = @source + (length - 1) * SizeOf(Character)
  For i = length To 1 Step - 1
    If Not FindString(#whitespace$, Chr(*ptrChar\c))
      ProcedureReturn Left(source, i)
    EndIf
    *ptrChar - SizeOf(Character)
  Next
EndProcedure
 
Procedure.s myTrim(source.s)
  ProcedureReturn myRTrim(myLTrim(source))
EndProcedure
 
If OpenConsole()
  PrintN(#DQUOTE$ + myLTrim("  Top  ") + #DQUOTE$)
  PrintN(#DQUOTE$ + myRTrim("  Tail  ") + #DQUOTE$)
  PrintN(#DQUOTE$ +  myTrim("  Both  ") + #DQUOTE$)
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit"): Input()
  CloseConsole()
EndIf

; Output:
; "Top  "
; "  Tail"
; "Both"
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 15
; Folding = -
; EnableXP
; EnableUnicode