﻿;==================================================================
; Rosetta Task: Synchronous concurrency
;
; The goal of this task is to create two concurrent activities ("Threads" or "Tasks", not processes.) that 
; share data synchronously. Your language may provide syntax or libraries to perform concurrency. Different
; languages provide different implementations of concurrency, often with different names. Some languages
; use the term threads, others use the term tasks, while others use co-processes. This task should not be 
; implemented using fork, spawn, or the Linux/UNIX/Win32 pipe command, as communication should be 
; between threads, not processes.
; 
; One of the concurrent units will read from a file named "input.txt" and send the contents of that file, one 
; line at a time, to the other concurrent unit, which will print the line it receives to standard output. The 
; printing unit must count the number of lines it prints. After the concurrent unit reading the file sends its
; last line to the printing unit, the reading unit will request the number of lines printed by the printing unit. 
; The reading unit will then print the number of lines printed by the printing unit.
; 
; This task requires two-way communication between the concurrent units. All concurrent units must 
; cleanly terminate at the end of the program.
;==================================================================

; PureBasic uses Semaphores and Mutex's to coordinate threads.

Enumeration 
  #Write
  #Done
EndEnumeration
 
Structure commblock
  txtline.s
  Order.i
EndStructure
 
Global MessageSent=CreateSemaphore()
Global LineWritten=CreateSemaphore()
Global LinesWritten, com.commblock
 
Procedure Writer(arg)
  Repeat 
    WaitSemaphore(MessageSent)
    If com\Order=#Write
      PrintN(com\txtline)
      LinesWritten+1
    EndIf 
    SignalSemaphore(LineWritten)
  Until com\Order=#Done
EndProcedure
 
Procedure Reader(arg)
  Protected File=ReadFile(#PB_Any,OpenFileRequester("","input.txt","",0))
  While file And Not Eof(file)
    com\txtline=ReadString(File)
    com\Order=#Write
    SignalSemaphore(MessageSent)
    WaitSemaphore(LineWritten)
  Wend
  com\Order=#Done
  SignalSemaphore(MessageSent)
  WaitSemaphore(LineWritten)
  PrintN(Str(LinesWritten)+" lines written.")
EndProcedure
 
If OpenConsole()
  Define Thread1=CreateThread(@Reader(),0)
  Define Thread2=CreateThread(@Writer(),0)
  Bool(WaitThread(Thread1) And WaitThread(Thread2))
  Print("Press Enter to exit"):Input()
EndIf
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 20
; Folding = -
; EnableXP
; EnableUnicode