﻿;==================================================================
; Rosetta Task: Trabb Pardo–Knuth algorithm
;
; The TPK algorithm is an early example of a programming chrestomathy. It was used in Donald Knuth 
; and Luis Trabb Pardo's Stanford tech report The Early Development of Programming Languages. The 
; report traces the early history of work in developing computer languages in the 1940s and 1950s, 
; giving several translations of the algorithm.
;==================================================================

Procedure.d f(x.d)
  ProcedureReturn Pow(Abs(x), 0.5) + 5 * x * x * x
EndProcedure
 
Procedure split(i.s, delimeter.s, List o.d())
  Protected index = CountString(i, delimeter) + 1 ;add 1 because last entry will not have a delimeter
 
  While index > 0
    AddElement(o())
    o() = ValD(Trim(StringField(i, index, delimeter)))
    index - 1
  Wend
 
  ProcedureReturn ListSize(o())
EndProcedure
 
Define i$, entriesAreValid = 0, result.d, output$
NewList numbers.d()
 
If OpenConsole()
  Repeat 
    PrintN(#CRLF$ + "Enter eleven numbers that are each separated by spaces or commas:")
 
    i$ = Input()
    i$ = Trim(i$)
    If split(i$, ",", numbers.d()) < 11
      ClearList(numbers())
      If split(i$, " ", numbers.d()) < 11
        PrintN("Not enough numbers were supplied.")
        ClearList(numbers())
      Else 
        entriesAreValid = 1
      EndIf 
    Else
      entriesAreValid = 1
    EndIf
  Until entriesAreValid = 1
 
  ForEach numbers()
    output$  = "f(" + RTrim(RTrim(StrD(numbers(), 3), "0"), ".") + ") = "
    result.d = f(numbers())
    If result > 400
      output$ + "Too Large"
    Else
      output$ + RTrim(RTrim(StrD(result, 3), "0"), ".")
    EndIf
    PrintN(output$)
  Next
 
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit"): Input()
  CloseConsole()
EndIf

; Output:
; Enter eleven numbers that are each separated by spaces Or commas:
; 10, -1, 1, 2, 3, 4, 4.3, 4.305, 4.303, 4.302, 4.301
; f(4.301) = 399.886
; f(4.302) = Too Large
; f(4.303) = Too Large
; f(4.305) = Too Large
; f(4.3) = 399.609
; f(4) = 322
; f(3) = 136.732
; f(2) = 41.414
; f(1) = 6
; f(-1) = -4
; f(10) = Too Large
; IDE Options = PureBasic 5.42 Beta 1 LTS (Windows - x86)
; CursorPosition = 32
; FirstLine = 28
; Folding = -
; EnableUnicode
; EnableXP