﻿;==================================================================
; Rosetta Task: Unbias a random generator
;
; Use your language's random number generator to create a function/method/subroutine/... randN that 
; returns a one or a zero, but with one occurring, on average, 1 out of N times, where N is an integer 
; from the range 3 to 6 inclusive.
;
; Create a function unbiased that uses only randN as its source of randomness to become an unbiased 
; generator of random ones and zeroes.
; For N over its range, generate and show counts of the outputs of randN and unbiased(randN).
;
; The actual unbiasing should be done by generating two numbers at a time from randN and only returning 
; a 1 or 0 if they are different. As long as you always return the first number or always return the second 
; number, the probabilities discussed above should take over the biased probability of randN.
;==================================================================

Procedure biased(n)
  If Random(n) <> 1
    ProcedureReturn 0
  EndIf 
  ProcedureReturn 1
EndProcedure
 
Procedure unbiased(n)
  Protected a, b
  Repeat
    a = biased(n)
    b = biased(n)
  Until a <> b
  ProcedureReturn a
EndProcedure
 
#count = 100000
 
Define n, m, output.s
For n = 3 To 6
  Dim b_count(1)
  Dim u_count(1)
  For m = 1 To #count
    x = biased(n)
    b_count(x) + 1
    x = unbiased(n)
    u_count(x) + 1
  Next
  output + "N = " + Str(n) + #LF$
  output + "  biased =>" + #TAB$ + "#0=" + Str(b_count(0)) + #TAB$ + "#1=" +Str(b_count(1))
  output + #TAB$ + " ratio=" + StrF(b_count(1) / #count * 100, 2) + "%" + #LF$
  output + "  unbiased =>" + #TAB$ + "#0=" + Str(u_count(0)) + #TAB$ + "#1=" + Str(u_count(1))
  output + #TAB$ + " ratio=" + StrF(u_count(1) / #count * 100, 2) + "%" + #LF$
Next
MessageRequester("Biased and Unbiased random number results", output)

; Sample output:
; 
; ---------------------------
; Biased And Unbiased random number results
; ---------------------------
; N = 3
;   biased =>	#0=74856	#1=25144	 ratio=25.14%
;   unbiased =>	#0=50066	#1=49934	 ratio=49.93%
; N = 4
;   biased =>	#0=80003	#1=19997	 ratio=20.00%
;   unbiased =>	#0=49819	#1=50181	 ratio=50.18%
; N = 5
;   biased =>	#0=83256	#1=16744	 ratio=16.74%
;   unbiased =>	#0=50268	#1=49732	 ratio=49.73%
; N = 6
;   biased =>	#0=85853	#1=14147	 ratio=14.15%
;   unbiased =>	#0=49967	#1=50033	 ratio=50.03%
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 12
; Folding = -
; EnableXP
; EnableUnicode