﻿;==================================================================
; Rosetta Task: Verify distribution uniformity - Naive
;
; This task is an adjunct to seven-sided dice from five-sided dice.
; 
; Create a function to check that the random integers returned from a small-integer generator function 
; have uniform distribution.
; 
; The function should take as arguments:
; 
; The function (or object) producing random integers.
; The number of times to call the integer generator.
; A 'delta' value of some sort that indicates how close to a flat distribution is close enough.
; The function should produce:
; 
; Some indication of the distribution achieved.
; An 'error' if the distribution is not flat enough.
; Show the distribution checker working when the produced distribution is flat enough and when it is not. 
; (Use a generator from seven-sided dice from five-sided dice).
;==================================================================

Prototype RandNum_prt()
 
Procedure.s distcheck(*function.RandNum_prt, repetitions, delta.d)
  Protected text.s, maxIndex = 0
  Dim bucket(maxIndex) ;array will be resized as needed
 
  For i = 1 To repetitions ;populate buckets
    v = *function()
    If v > maxIndex
      maxIndex = v
      ReDim bucket(maxIndex)
    EndIf 
    bucket(v) + 1
  Next 
 
 
  lbnd = Round((repetitions / maxIndex) * (100 - delta) / 100, #PB_Round_Up)
  ubnd = Round((repetitions / maxIndex) * (100 + delta) / 100, #PB_Round_Down)
  text = "Distribution check:" + #CRLF$ + #CRLF$
  text + "Total elements = " + Str(repetitions) + #CRLF$ + #CRLF$
  text + "Margin = " + StrF(delta, 2) + "% --> Lbound = " + Str(lbnd) + ", Ubound = " + Str(ubnd) + #CRLF$
 
  For i = 1 To maxIndex 
    If bucket(i) < lbnd Or bucket(i) > ubnd
      text + #CRLF$ + "Bucket " + Str(i) + " contains " + Str(bucket(i)) + " elements.  Skewed."
    EndIf 
  Next 
  ProcedureReturn text
EndProcedure
 
MessageRequester("Results", distcheck(@dice7(), 1000000, 0.20))

; A small delta was chosen To increase the chance of a skewed result in the sample output:
; 
; Distribution check:
; 
; Total elements = 1000000
; 
; Margin = 0.20% --> Lbound = 142572, Ubound = 143142
; 
; Bucket 1 contains 141977 elements.  Skewed.
; Bucket 6 contains 143860 elements.  Skewed.
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 18
; Folding = -
; EnableXP
; EnableUnicode