﻿;==================================================================
; Rosetta Task: Word wrap
;
; Even today, with proportional fonts and complex layouts, there are still cases where you need to wrap 
; text at a specified column.
; 
; The basic task is to wrap a paragraph of text in a simple way in your language.
; 
; If there is a way to do this that is built-in, trivial, or provided in a standard library, show that. Otherwise
; implement the minimum length greedy algorithm from Wikipedia.
; 
; Show your routine working on a sample of text at two different wrap columns.
; 
; Extra credit! Wrap text using a more sophisticated algorithm such as the Knuth and Plass TeX algorithm. 
; If your language provides this, you get easy extra credit, but you must reference documentation indicating 
; that the algorithm is something better than a simple minimimum length algorithm.
; 
; If you have both basic and extra credit solutions, show an example where the two algorithms give different results.
;==================================================================

DataSection  
  Data.s "In olden times when wishing still helped one, there lived a king "+
	"whose daughters were all beautiful, but the youngest was so beautiful "+
	"that the sun itself, which has seen so much, was astonished whenever "+
	"it shone-in-her-face.  Close-by-the-king's castle lay a great dark "+
	"forest, and under an old lime-tree in the forest was a well, and when "+
	"the day was very warm, the king's child went out into the forest and "+
	"sat down by the side of the cool-fountain, and when she was bored she "+
	"took a golden ball, and threw it up on high and caught it, and this "+
	"ball was her favorite plaything."
EndDataSection
 
Procedure.i ww_pos(txt$,l.i)
  While Mid(txt$,l,1)<>Chr(32) And l>0 And Len(txt$)>l : l-1 : Wend
  If l>0 : ProcedureReturn l : Else : ProcedureReturn Len(Trim(txt$)) : EndIf
EndProcedure
 
Procedure WriteLine(txt$,ls.i) 
  Shared d$,lw
  Select LCase(d$)
    Case "l" : PrintN(Mid(txt$,1,ls))    
    Case "r" : PrintN(RSet(Trim(Mid(txt$,1,ls)),lw,Chr(32)))
  EndSelect  
EndProcedure
 
Procedure main(txt$,lw.i)
  If Len(txt$)
    p=ww_pos(txt$,lw) : WriteLine(txt$,p) : ProcedureReturn main(LTrim(Right(txt$,Len(txt$)-p)),lw)
  EndIf
EndProcedure
 
Procedure.i MaxWordLen(txt$)
  For i=1 To CountString(txt$,Chr(32))+1
    wrd$=LTrim(StringField(txt$,i,Chr(32)))
    wrdl=Len(wrd$)+1 : If wrdl>l : l=wrdl : EndIf    
  Next
  ProcedureReturn l
EndProcedure
 
OpenConsole()
Read.s t$
Print("Input line width: ") : lw=Val(Input()) : minL=MaxWordLen(t$)
If lw<minL : lw=minL : PrintN("Min. line width "+Str(lw-1)) : EndIf
Print("Input direction l:left r:rigth ")
Repeat : d$=Inkey() : Delay(50) : Until FindString("lr",d$,1,#PB_String_NoCase) : PrintN(d$+#CRLF$)
main(t$,lw) : Input()
 
; Output:
; Input line width: 40
; Input direction l:left r:rigth l
; 
; In olden times when wishing still
; helped one, there lived a king whose
; daughters were all beautiful, but the
; youngest was so beautiful that the sun
; itself, which has seen so much, was
; astonished whenever it
; shone-in-her-face.  Close-by-the-king's
; castle lay a great dark forest, And
; under an old lime-tree in the forest
; was a well, And when the day was very
; warm, the king's child went out into
; the forest And sat down by the side of
; the cool-fountain, And when she was
; bored she took a golden ball, And threw
; it up on high And caught it, And this
; ball was her favorite plaything.
; Input line width: 40
; Input direction l:left r:rigth r
; 
;        In olden times when wishing still
;     helped one, there lived a king whose
;    daughters were all beautiful, but the
;   youngest was so beautiful that the sun
;      itself, which has seen so much, was
;                   astonished whenever it
;  shone-in-her-face.  Close-by-the-king's
;      castle lay a great dark forest, And
;     under an old lime-tree in the forest
;    was a well, And when the day was very
;     warm, the king's child went out into
;   the forest And sat down by the side of
;      the cool-fountain, And when she was
;  bored she took a golden ball, And threw
;    it up on high And caught it, And this
;         ball was her favorite plaything.
; IDE Options = PureBasic 5.50 beta 3 (Windows - x64)
; CursorPosition = 17
; Folding = -
; EnableXP
; EnableUnicode