﻿;==================================================================
; Rosetta Task: XML - Output
;
; Create a function that takes a list of character names and a list of corresponding remarks and returns an 
; XML document of <Character> elements each with a name attributes and each enclosing its remarks. 
; All <Character> elements are to be enclosed in turn, in an outer <CharacterRemarks> element.
; 
; As an example, calling the function with the three names of:
;    April
;    Tam O'Shanter
;    Emily
; And three remarks of:
;    Bubbly: I'm > Tam and <= Emily
;    Burns: "When chapman billies leave the street ..."
;    Short & shrift
;
; Should produce the XML (but not necessarily with the indentation):
; 
; <CharacterRemarks>
;     <Character name="April">Bubbly: I'm &gt; Tam and &lt;= Emily</Character>
;     <Character name="Tam O'Shanter">Burns: "When chapman billies leave the street ..."</Character>
;     <Character name="Emily">Short &amp; shrift</Character>
; </CharacterRemarks>
;
; The document may include an <?xml?> declaration and document type declaration, but these are optional. 
; If attempting this task by direct string manipulation, the implementation must include code to perform 
; entity substitution for the characters that have entities defined in the XML 1.0 specification.
; 
; Note: the example is chosen to show correct escaping of XML strings. Note too that although the task is 
; written to take two lists of corresponding data, a single mapping/hash/dictionary of names to remarks is also acceptable.
; 
; Note to editors: Program output with escaped characters will be viewed as the character on the page so 
; you need to 'escape-the-escapes' to make the RC entry display what would be shown in a plain text viewer (See this). 
; Alternately, output can be placed in <lang xml></lang> tags without any special treatment.
;==================================================================

DataSection
  dataItemCount:
  Data.i 3
  
  names:
  Data.s "April", "Tam O'Shanter", "Emily"
  
  remarks:
  Data.s "Bubbly: I'm > Tam and <= Emily",
         ~"Burns: \"When chapman billies leave the street ...\"",
         "Short & shrift"
EndDataSection

Structure characteristic
  name.s
  remark.s
EndStructure  

NewList didel.characteristic()
Define item.s, numberOfItems, i

Restore dataItemCount
Read.i numberOfItems

;add names
Restore names
For i = 1 To numberOfItems
  AddElement(didel())
  Read.s item
  didel()\name = item
Next

;add remarks
ResetList(didel())
FirstElement(didel())
Restore remarks:
For i = 1 To numberOfItems
  Read.s item
  didel()\remark = item
  NextElement(didel())
Next

Define xml, mainNode, itemNode
ResetList(didel())
FirstElement(didel())
xml = CreateXML(#PB_Any) 
mainNode = CreateXMLNode(RootXMLNode(xml), "CharacterRemarks") 
ForEach didel()
  itemNode = CreateXMLNode(mainNode, "Character")
  SetXMLAttribute(itemNode, "name", didel()\name) 
  SetXMLNodeText(itemNode, didel()\remark) 
Next
FormatXML(xml, #PB_XML_ReFormat |  #PB_XML_WindowsNewline | #PB_XML_ReIndent)

If OpenConsole()
  PrintN(ComposeXML(xml))
  Print(#CRLF$ + #CRLF$ + "Press ENTER to exit"): Input()
  CloseConsole()
EndIf

; IDE Options = PureBasic 5.41 LTS (Windows - x64)
; CursorPosition = 94
; FirstLine = 41
; EnableUnicode
; EnableXP